#! /usr/bin/perl
#
#  Purpose:
#  FlowGrapher.cgi creates the FlowGrapher web page for inputting
#  selection criteria.
#
#  Description:
#
#  FlowGrapher.cgi will create a form for accepting parameters to control
#  the filtering and selection of Netflow data in the generation of a
#  flow graph.
#
#  Input arguments:
#  Name                 Description
#  -----------------------------------------------------------------------
#  none      
#
#  Modification history:
#  Author       Date            Vers.   Description
#  -----------------------------------------------------------------------
#  J. Loiacono  01/01/2006      2.0     Original distributed version.
#  J. Loiacono  01/26/2006      2.2     Added flow_select option
#  J. Loiacono  07/04/2006      3.0     Replaces create_FG_webpage
#  J. Loiacono  12/25/2006      3.1     New 300 second sample time
#  J. Loiacono  02/22/2007      3.2     [No Change to this module]
#  J. Loiacono  12/07/2007      3.3     Named Interfaces (thanks C. Kashimoto) 
#                                       Saved Filters, Next Hop
#
#$Author$
#$Date$
#$Header$
#
###########################################################################
#
#               BEGIN EXECUTABLE STATEMENTS
#
 
use FlowViewer_Configuration;
use FlowViewer_Utilities;

if ($debug_grapher eq "Y") { open (DEBUG,">$work_directory/DEBUG_GRAPHER"); }

# Create the header portion of the FlowGrapher webpage

print "Content-type:text/html\n\n"; 
print "<html>\n";
print "<head>\n";
print "<title>\n";
print "FlowGrapher $version\n";
print "</title>\n";
print "</head>\n";
print "<body bgcolor=$bg_color text=$text_color link=$link_color vlink=$vlink_color>";
print "<center>";
print "<img src=$graphs_short/FlowGrapher.png border=0 usemap=#FlowGrapher_Map>"; 
if ($user_logo ne "") {  
        print "<a href=$user_hyperlink>"; 
        print "<img src=$reports_short/$user_logo border=0>"; 
        print "</a>";  
}     
print "</center><b><pre>";

# Copy image into Graphs directory 
      
if (!-e $graphs_directory) {

        mkdir($graphs_directory,$html_dir_perms) || die "cannot mkdir $graphs_directory: $!";
        chmod $html_dir_perms, $graphs_directory;

        $copy_command = "cp $cgi_bin_directory/FlowGrapher.png $graphs_directory";
        system($copy_command);
        $icon_png_file = "$graphs_directory/FlowGrapher.png"; 
        chmod $html_dir_perms, $icon_png_file; 

        $copy_command = "cp $cgi_bin_directory/FlowGrapher_Save.png $graphs_directory";
        system($copy_command);
        $icon_png_file = "$graphs_directory/FlowGrapher_Save.png"; 
        chmod $html_dir_perms, $icon_png_file; 

        print "<pre><b>";
        print "      The directory for storing FlowGrapher files has been created:\n\n";
        print "      <i>$graphs_directory</i>\n\n";
        print "      Please ensure this directory has adequate permissions for your\n";
        print "      web server process owner (e.g., 'apache') to write into it.\n\n";
        print "\n";
        print "      <a href=$cgi_bin_short/FlowGrapher.cgi>RETURN</a>\n";
        print "</pre></b>";

        exit;
}

$copy_command = "cp $cgi_bin_directory/FlowGrapher.png $graphs_directory"; 
system($copy_command); 

$copy_command = "cp $cgi_bin_directory/FlowGrapher_Save.png $graphs_directory"; 
system($copy_command); 

# Set up start and end times according to configurable start and end offsets

($sec,$min,$hr,$date,$mnth,$yr,$day,$yr_date,$DST) = localtime(time-$start_offset);
$mnth += 1;
$yr   += 1900;
$min_delta = $min % 5;
$min = $min - $min_delta;
if (length $min < 2) { $min = "0" . $min; }
$start_date = $mnth ."/". $date ."/". $yr;
if ($use_even_hours eq "Y") { 
        $start_time = $hr .":00:00"; } 
else { 
        $start_time = $hr .":". $min .":00"; } 

($sec,$min,$hr,$date,$mnth,$yr,$day,$yr_date,$DST) = localtime(time-$end_offset);
$mnth += 1;
$yr   += 1900;
$min_delta = $min % 5;
$min = $min - $min_delta;
if (length $min < 2) { $min = "0" . $min; }
$end_date = $mnth ."/". $date ."/". $yr;
if ($use_even_hours eq "Y") { 
        $end_time = $hr .":00:00"; } 
else { 
        $end_time = $hr .":". $min .":00"; } 

# If a user has called this program from another FlowViewer tool, pre-fill the input fields

$query_string = $ENV{'QUERY_STRING'};

if ($query_string ne "") {
	$query_string =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/ge ;
	chop $query_string;
	@fields = split(/&/,$query_string);
	foreach $input_field (@fields) {
		($field,$field_value) = split(/=/,$input_field);
                if ($field eq "filter_hash")         { $filter_hash = $field_value; }
                if ($field eq "device_name")         { $selected_device_name = $field_value; }
                if ($field eq "start_date")          { $start_date = $field_value; }
                if ($field eq "start_time")          { $start_time = $field_value; }
                if ($field eq "end_date")            { $end_date = $field_value; }
                if ($field eq "end_time")            { $end_time = $field_value; }
                if ($field eq "source_addresses")    { $source_address = $field_value; }
                if ($field eq "source_ports")        { $source_port = $field_value; }
                if ($field eq "source_ifs")          { $source_if = $field_value; }
                if ($field eq "sif_names")           { $selected_sif_name = $field_value; }
                if ($field eq "source_ases")         { $source_as = $field_value; }
                if ($field eq "dest_addresses")      { $dest_address = $field_value; }
                if ($field eq "dest_ports")          { $dest_port = $field_value; }
                if ($field eq "dest_ifs")            { $dest_if = $field_value; }
                if ($field eq "dif_names")           { $selected_dif_name = $field_value; }
                if ($field eq "dest_ases")           { $dest_as = $field_value; }
                if ($field eq "protocols")           { $protocols = $field_value; }
                if ($field eq "tos_fields")          { $tos_fields = $field_value; }
                if ($field eq "tcp_flags")           { $tcp_flags = $field_value; }
                if ($field eq "exporter")            { $selected_exporter = $field_value; }
                if ($field eq "nexthop_ips")         { $nexthop_ip = $field_value; }
                if ($field eq "tracking_label")      { $tracking_label = $field_value; }
                if ($field eq "general_comment")     { $general_comment = $field_value; }
                if ($field eq "sampling_multiplier") { $sampling_multiplier = $field_value; }
        }
}

if ($selected_device_name ne "") {
        $interfaces_file = "$cgi_bin_directory/NamedInterfaces_Devices";
} elsif ($selected_exporter ne "") {
        $interfaces_file = "$cgi_bin_directory/NamedInterfaces_Exporters";
}

$detail_lines = 20;

print "<form action=$cgi_bin_short/FlowGrapher_Main.cgi method=POST>";
print "Filter Criteria:";

# Print out Saved Filters

$saved_filters_file = "$cgi_bin_directory/FlowViewer_SavedFilters"; 
if (-e $saved_filters_file) {
	$pulldown_width = 140;
	print " <select name=node onchange=window.location.href=this[selectedIndex].value\; style=\"width: $pulldown_width\">\n";
	if ($filter_hash eq "") { print "<option selected>Select Saved Filter"; }
	open (FILTERS,"<$saved_filters_file"); 
	while (<FILTERS>) { 
		chomp;
	        ($existing_hash,$existing_name,$existing_string) = split(/\^/); 
		$option_value = "$cgi_bin_short/FlowGrapher.cgi?filter_hash=$existing_hash&$existing_string";
		if ($existing_hash eq $filter_hash) {
			print "<option selected value=$option_value>$existing_name";
			$current_hash = $existing_hash;
		} else {
			print "<option value=$option_value>$existing_name";
		}
	}
	close FILTERS;
	$option_value = "$cgi_bin_short/FlowViewer_Save.cgi?name_filter^$current_hash";
	print "<option value=$option_value>Review or Modify Filters";
	print "</select>";
}
print "\n\n";

if ($devices[0] ne "") {
        print "         Device: <select name=device_name onchange=window.location.href=this[selectedIndex].value\; style=\"width: $pulldown_width\">\n";
        print "<option selected value=\"\">Select Device\n";
        foreach $device_name (@devices) {
                $option_value = "$cgi_bin_short/FlowGrapher.cgi?device_name=$device_name&";
                if ($device_name eq $selected_device_name) {
                        print "<option selected value=$option_value>$device_name";
                } else {
                        print "<option value=$option_value>$device_name";
                }
        }
        print "</select>";
}

if ($exporters[0] ne "") {
        if ($devices[0] ne "") { print "&nbsp&nbsp&nbsp&nbsp&nbsp"; }
        print "       Exporter: <select name=exporter onchange=window.location.href=this[selectedIndex].value\; style=\"width: $pulldown_width\">\n";
        print "<option selected value=\"\">Select Exporter\n";
        foreach $exporter_pair (@exporters) {
                ($exporter_ip,$exporter_name) = split(/:/,$exporter_pair);
                $option_value = "$cgi_bin_short/FlowGrapher.cgi?exporter=$exporter_ip&";
                if ($exporter_ip eq $selected_exporter) {
                        $exporter = $exporter_ip;
                        print "<option selected value=$option_value>$exporter_name";
                } else {
                        print "<option value=$option_value>$exporter_name";
                }
        }
        print "</select>";
}

if (($devices[0] ne "") && ($exporters[0] eq "")) {
        print "      Next Hop IP: <input type=text size=19 name=nexthop_ip value=\"$nexthop_ip\">"; }
if (($devices[0] eq "") && ($exporters[0] ne "")) {
        print "    Next Hop IP: <input type=text size=19 name=nexthop_ip value=\"$nexthop_ip\">"; }
if (($devices[0] ne "") && ($exporters[0] ne "")) {
        print "        Next Hop IP: <input type=text size=19 name=nexthop_ip value=\"$nexthop_ip\">"; }
if (($devices[0] eq "") && ($exporters[0] eq "")) {
        print "    Next Hop IP: <input type=text size=19 name=nexthop_ip value=\"$nexthop_ip\">"; }
print "\n\n";

print "     Start Date: <input type=text size=10 name=start_date value=$start_date></b> (mm/dd/yyyy)<b>    Start Time: <input type=text size=10 name=start_time value=$start_time>  </b>(hh:mm:ss)<b>     TOS Field: <input type=text size=10 name=tos_fields value=\"$tos_fields\">  </b>(e.g., -0x0b/0x0F)<b>\n";
print "\n";
print "       End Date: <input type=text size=10 name=end_date value=$end_date></b> (mm/dd/yyyy)<b>      End Time: <input type=text size=10 name=end_time value=$end_time>  </b>(hh:mm:ss)<b>      TCP Flag: <input type=text size=10 name=tcp_flags value=\"$tcp_flags\">    Protocol: <input type=text size=6 name=protocols value=\"$protocols\">\n";
print "\n";

# Print out pulldown with named Source interfaces

print "      Source IP: <input type=text size=19 name=source_address value=\"$source_address\">";
print "    Source Interface: <input type=text size=2 name=source_if value=\"$source_if\">";

# Print out Include named interfaces

open (NAMED,"<$interfaces_file");
chomp (@interfaces = <NAMED>);
close (NAMED);

print "  <select name=sif_name width=20></b>\n";
print "<option selected value=\"\">Interface Names\n";
print "<optgroup label=\"Include ...\">";
foreach $interface (@interfaces) {
        if (($interface eq "") || (substr($interface,0,1) eq "#")) { next; }
        ($device,$interface_index,$interface_name) = split(/:/,$interface);
        if (($device eq $selected_device_name) || ($device eq $selected_exporter)) {
                if ($interface_index eq $selected_sif_name) {
                        print "<option selected value=$interface_index>$interface_index $interface_name\n";
                } else {
                        print "<option value=$interface_index>$interface_index $interface_name\n";
                }
        }
}

# Print out Exclude named interfaces

print "<optgroup label=\"Exclude ...\">";
foreach $interface (@interfaces) {
        if ($interface eq "") { next; }
        ($device,$interface_index,$interface_name) = split(/:/,$interface);
        $interface_index *= -1;
        if (($device eq $selected_device_name) || ($device eq $selected_exporter)) {
                if ($interface_index eq $selected_sif_name) {
                        print "<option selected value=$interface_index>$interface_index $interface_name\n";
                } else {
                        print "<option value=$interface_index>$interface_index $interface_name\n";
                }
        }
}

print "</select>";
print "       Source Port: <input type=text size=5 name=source_port value=\"$source_port\">";
print "   Source AS: <input type=text size=6 name=source_as value=\"$source_as\">\n\n";

# Print out pulldown with named Destination interfaces

print "        Dest IP: <input type=text size=19 name=dest_address value=\"$dest_address\">";
print "      Dest Interface: <input type=text size=2 name=dest_if value=\"$dest_if\">";

# Print out Include named interfaces

open (NAMED,"<$interfaces_file");
chomp (@interfaces = <NAMED>);
close (NAMED);

print "  <select name=dif_name width=20></b>\n";
print "<option selected value=\"\">Interface Names\n";
print "<optgroup label=\"Include ...\">";
foreach $interface (@interfaces) {
        if ($interface eq "") { next; }
        ($device,$interface_index,$interface_name) = split(/:/,$interface);
        if (($device eq $selected_device_name) || ($device eq $selected_exporter)) {
                if ($interface_index eq $selected_dif_name) {
                        print "<option selected value=$interface_index>$interface_index $interface_name\n";
                } else {
                        print "<option value=$interface_index>$interface_index $interface_name\n";
                }
        }
}

# Print out Exclude named interfaces

print "<optgroup label=\"Exclude ...\">";
foreach $interface (@interfaces) {
        if ($interface eq "") { next; }
        ($device,$interface_index,$interface_name) = split(/:/,$interface);
        $interface_index *= -1;
        if (($device eq $selected_device_name) || ($device eq $selected_exporter)) {
                if ($interface_index eq $selected_dif_name) {
                        print "<option selected value=$interface_index>$interface_index $interface_name\n";
                } else {
                        print "<option value=$interface_index>$interface_index $interface_name\n";
                }
        }
}

print "</select>";
print "         Dest Port: <input type=text size=5 name=dest_port value=\"$dest_port\">";
print "     Dest AS: <input type=text size=6 name=dest_as value=\"$dest_as\">\n";

print "\n";
print "<center></b>Notes: Multiple field entries, separated by commas, are permitted in the fields above.\n";
print "A minus sign (-) will negate an entry (e.g. -1776 for AS, would mean any AS but 1776).\n";
print "IP fields accept networks (e.g. 192.168.10.0/19), hosts, and names (e.g. www.abc.com).<b></center>";
print "\n";
print "<hr size=4>";
print "Graphing Parameters:\n\n";
print "     Detail Lines: <input type=text size=6 name=detail_lines value=$detail_lines>";
print "    Graph Width: <select name=graph_multiplier width=2><option value=1>1<option value=2>2<option value=3>3<option value=4>4</select>"; 
print "            Sample Time: <select name=sample_time width=2><option value=1>1<option value=2>2<option value=3>3<option value=4>4<option value=5>5<option value=10>10<option value=15>15<option value=30>30<option value=60>60<option selected>5</select>";
print "        Resolve Addresses: <select name=resolve_addresses width=4></b>\n"; 
print "    <option selected value=Y>Y\n"; 
print "    <option value=N>N\n"; 
print "    </select>";
print "\n\n";
print "  Include Flow if: <select name=flow_select width=16></b>\n"; 
print "<option value=1 selected>Any Part in Specified Time Span\n"; 
print "<option value=2>End Time in Specified Time Span\n"; 
print "<option value=3>Start Time in Specified Time Span\n"; 
print "<option value=4>Entirely in Specified Time Span\n"; 
print "</select>";
print "     Sampling Multiplier: <input type=text size=3 name=sampling_multiplier value=$sampling_multiplier>";
print "    Graph Type: <select name=graph_type width=8></b>\n";
print "    <option value=Bits selected>Bits/second\n";
print "    <option value=Flows>Flows/second\n";
print "    <option value=Packets>Packets/second\n";
print "    </select>";
print "\n\n";
print "<hr size=4>";
print "\n\n";
print "<center>";
print "<input type=submit name=action value=\"Generate Graph\">&nbsp\;<input type=reset value=\"Reset Values\">\n";
print "</center>";
print "</pre>";
print "</form>\n";
print "<map name=FlowGrapher_Map>\n";  
print "<area shape=rect coords=\"0,0,75,88\" href=$cgi_bin_short/FlowGrapher.cgi?$call_string>\n";  
print "<area shape=rect coords=\"75,22,271,68\" href=$cgi_bin_short/FlowGrapher.cgi?$call_string>\n";  
print "<area shape=rect coords=\"75,0,271,22\" href=$cgi_bin_short/FlowViewer.cgi?$call_string>\n";  
print "<area shape=rect coords=\"75,68,271,88\" href=$cgi_bin_short/FlowTracker.cgi?$call_string>\n";  
print "</map>\n";
print "</body>\n";
print "</html>\n";
