#! /usr/bin/perl
#
#  Purpose:
#  FlowGrapher_Sort.cgi sorts FlowGrapher detail line output and
#  redisplays the page.
#
#  Description:
#  The script is invoked by the user clicking on a column heading of the
#  Detail Lines section of the FlowGgrapher output page. A temporary file
#  that contains a copy of the detail lines is read in and sorted. A new page
#  is output.
#
#  Input arguments (received from the form):
#  Name                 Description
#  -----------------------------------------------------------------------
#  sort_column          Identifies which column to sort on
#  file_index           Identifies which temporary file to use
#
#  Modification history:
#  Author       Date            Vers.   Description
#  -----------------------------------------------------------------------
#  J. Loiacono  12/07/2007      3.3     Original released version
#  J. Loiacono  12/28/2007      3.3.1   Fixed sorting of host names
#
#$Author$
#$Date$
#$Header$
#
###########################################################################
#
#               BEGIN EXECUTABLE STATEMENTS
#
 
use FlowViewer_Configuration;
use FlowViewer_Utilities;

if ($debug_grapher eq "Y") { open (DEBUG,">$work_directory/DEBUG_GRAPHER"); }

# Retrieve the form inputs
 
($sort_column,$file_index,$ascend) = split(/\^/,$ENV{'QUERY_STRING'});

if ($sort_column eq "Start")   { $sort_column = 1; }
if ($sort_column eq "End")     { $sort_column = 2; }
if ($sort_column eq "Len")     { $sort_column = 3; }
if ($sort_column eq "Source")  { $sort_column = 4; }
if ($sort_column eq "Sport")   { $sort_column = 5; }
if ($sort_column eq "Dest")    { $sort_column = 6; }
if ($sort_column eq "Dport")   { $sort_column = 7; }
if ($sort_column eq "Bytes")   { $sort_column = 8; }
if ($sort_column eq "Mbps")    { $sort_column = 9; }

# Load the temporary file of detail lines

$sort_file = "$work_directory/FlowGrapher_sort_" . $file_index;
open(SORT,"<$sort_file");
chomp (@sort_lines = <SORT>);
close SORT;

foreach $sort_line (@sort_lines) {

	$line_count++;

	if ($line_count == 1) { $flowgraph_link      = $sort_line; next; }
	if ($line_count == 2) { $save_filename       = $sort_line; next; }
	if ($line_count == 3) { $call_string         = $sort_line; next; }
	if ($line_count == 4) { $graph_type          = (split(/: /,$sort_line)) [1]; next; }
	if ($line_count == 5) { $resolve_addresses   = (split(/: /,$sort_line)) [1]; next; }
	if ($line_count == 6) { $sampling_multiplier = (split(/: /,$sort_line)) [1]; next; }
	
	if ($sort_line =~ /begin_parameters/) { $parameters = "on"; next; }
	if ($parameters eq "on") {
		if ($sort_line =~ /end_parameters/) { 
			$parameters = "done";
		} else {
			push (@parameter_lines,$sort_line);
		}
		next;
	}

	$sort_value = (split(/\s+/,$sort_line)) [$sort_column];

	if (($sort_column == 4) || ($sort_column == 6)) {
		if (!($sort_value =~ /[a-zA-Z]/)) {
		        ($a,$b,$c,$d) = split(/\./,$sort_value); 
		        if (length($a) == 1) { $a = "00" . $a; } if (length($a) == 2) { $a = "0" . $a; }  
		        if (length($b) == 1) { $b = "00" . $b; } if (length($b) == 2) { $b = "0" . $b; }  
		        if (length($c) == 1) { $c = "00" . $c; } if (length($c) == 2) { $c = "0" . $c; }  
		        if (length($d) == 1) { $d = "00" . $d; } if (length($d) == 2) { $d = "0" . $d; }  
		        $sort_value = $a . $b . $c . $d;
		}
		$sort_key = $sort_value;
	} else {
	        $len_sort = length($sort_value); 
	        $sort_key = $sort_value; 
	        for ($i=$len_sort;$i<22;$i++) { 
	                $sort_key = "0" . $sort_key; 
	        }
	}

	$presort_line = $sort_key ."  ". $sort_line;
	push(@presort_lines,$presort_line);

	$detail_lines++;
}

@sorted_lines = sort (@presort_lines);

if ($ascend == 0) {
	if (($sort_column == 8) || ($sort_column == 9) || ($sort_column == 3)) { 
		@sorted_temp = reverse(@sorted_lines); 
		@sorted_lines = @sorted_temp;
	}
	$ascend = 1;
} else {
	if (($sort_column != 8) && ($sort_column != 9) && ($sort_column != 3)) { 
		@sorted_temp = reverse(@sorted_lines); 
		@sorted_lines = @sorted_temp;
	}
	$ascend = 0;
}

# Start the web page output and the save file output
 
$last_slash = rindex($save_filename,"/");
$save_filename = substr($save_filename,$last_slash,200);
$save_fullname = "$work_directory/$save_filename";
open (HTML,">$save_fullname"); 
chmod $html_file_perms, $save_filename;
     
print "Content-type:text/html\n\n";
print "<head>\n";
print "<style type=text/css><!-- A { text-decoration:none } --></style>\n";
print "</head>\n";
print "<body bgcolor=$bg_color>\n";
print "<title>FlowGrapher $version Graph Results</title>\n";
print "<center>\n";
 
print "<table>\n"; 
print "<tr>\n"; 
print "<td align=center>\n"; 
print "<a href=$cgi_bin_short/FlowGrapher.cgi><img src=$graphs_short/FlowGrapher_Save.png border=0 usemap=#FlowGrapherSave_Map></a>\n";
if ($user_logo ne "") {     
        print "<a href=$user_hyperlink>\n";   
        print "<img src=$reports_short/$user_logo border=0>\n";   
        print "</a>\n";    
}    
print "</td>\n"; 
print "</tr>\n";
print "</table>\n";

print HTML "<head>\n";
print HTML "<style type=text/css><!-- A { text-decoration:none } --></style>\n";
print HTML "</head>\n";
print HTML "<body bgcolor=$bg_color>\n";
print HTML "<title>FlowGrapher $version Saved Graph Results</title>\n";
print HTML "<center>\n";
print HTML "<table>\n";
print HTML "<tr>\n";
print HTML "<td></td>\n";
print HTML "<td align=center>\n";
print HTML "<a href=$cgi_bin_short/FlowGrapher.cgi><img src=$graphs_short/FlowGrapher.png border=0 usemap=#FlowGrapher_Map></a>\n";
if ($user_logo ne "") {    
        print HTML "<a href=$user_hyperlink>";  
        print HTML "<img src=$reports_short/$user_logo border=0>";  
        print HTML "</a>";   
} 

print HTML "<p>\n";
print HTML "</td>\n";
print HTML "</tr>\n";
print HTML "</table><p><table>\n";

print "<table>\n";
foreach $parameter (@parameter_lines) { print "$parameter\n"; print HTML "$parameter\n"; }
	
# Output the graph

print "<center><img src=$flowgraph_link></center>\n";
print HTML "<center><img src=$flowgraph_link></center>\n";

# Output the column headers

print "<br><pre><b>";
print HTML "<br><pre><b>";

if ($graph_type eq "Packets") {
	$totals_txt = "Total Packets";
	$ps_txt = "Kpps";
} elsif ($graph_type eq "Flows") {
	$totals_txt = "Total Flows";
	$ps_txt = "Kfps";
} else {
	$totals_txt = "Total Bytes";
	$ps_txt = "Mbps";
}

if ($sampling_multiplier > 1) { 
	$totals_txt .= "*";
	$ps_txt     .= "*";
}

if ($resolve_addresses eq "Y") {
        $start_link  = "<a href=$cgi_bin_short/FlowGrapher_Sort.cgi?Start^$file_index^$ascend>Start    </a>";
        $end_link    = "<a href=$cgi_bin_short/FlowGrapher_Sort.cgi?End^$file_index^$ascend> End</a>";
        $len_link    = "<a href=$cgi_bin_short/FlowGrapher_Sort.cgi?Len^$file_index^$ascend>         Len</a>";
        $source_link = "<a href=$cgi_bin_short/FlowGrapher_Sort.cgi?Source^$file_index^$ascend>  Source Host</a>";
        $sport_link  = "<a href=$cgi_bin_short/FlowGrapher_Sort.cgi?Sport^$file_index^$ascend>                Port</a>";
        $dest_link   = "<a href=$cgi_bin_short/FlowGrapher_Sort.cgi?Dest^$file_index^$ascend>     Destination Host</a>";
        $dport_link  = "<a href=$cgi_bin_short/FlowGrapher_Sort.cgi?Dport^$file_index^$ascend>           Port</a>";
        if ($graph_type eq "Packets") {
                $bytes_link  = "<a href=$cgi_bin_short/FlowGrapher_Sort.cgi?Bytes^$file_index^$ascend>     $totals_txt</a>";
        } else {
                $bytes_link  = "<a href=$cgi_bin_short/FlowGrapher_Sort.cgi?Bytes^$file_index^$ascend>       $totals_txt</a>";
        }
        if ($sampling_multiplier > 1) {
                $mbps_link   = "<a href=$cgi_bin_short/FlowGrapher_Sort.cgi?Mbps^$file_index^$ascend>   $ps_txt</a>";
        } else {
                $mbps_link   = "<a href=$cgi_bin_short/FlowGrapher_Sort.cgi?Mbps^$file_index^$ascend>    $ps_txt</a>";
        }
        print "$start_link$end_link$len_link$source_link$sport_link$dest_link$dport_link$bytes_link$mbps_link</b>\n\n";
        printf HTML "%-9s %-9s %6s %-25s  %-6s   %-25s  %-6s  %14s  %7s\n\n</b>", "Start", "End", "Len", "Source Host", "Port", "Destination Host", "Port", $totals_txt, $ps_txt;
} else {
        $start_link  = "<a href=$cgi_bin_short/FlowGrapher_Sort.cgi?Start^$file_index^$ascend>Start    </a>";
        $end_link    = "<a href=$cgi_bin_short/FlowGrapher_Sort.cgi?End^$file_index^$ascend> End</a>";
        $len_link    = "<a href=$cgi_bin_short/FlowGrapher_Sort.cgi?Len^$file_index^$ascend>         Len</a>";
        $source_link = "<a href=$cgi_bin_short/FlowGrapher_Sort.cgi?Source^$file_index^$ascend>  Source Host</a>";
        $sport_link  = "<a href=$cgi_bin_short/FlowGrapher_Sort.cgi?Sport^$file_index^$ascend>           Port</a>";
        $dest_link   = "<a href=$cgi_bin_short/FlowGrapher_Sort.cgi?Dest^$file_index^$ascend>     Destination Host</a>";
        $dport_link  = "<a href=$cgi_bin_short/FlowGrapher_Sort.cgi?Dport^$file_index^$ascend>      Port</a>";
        if ($graph_type eq "Packets") {
                $bytes_link  = "<a href=$cgi_bin_short/FlowGrapher_Sort.cgi?Bytes^$file_index^$ascend>     $totals_txt</a>";
        } else {
                $bytes_link  = "<a href=$cgi_bin_short/FlowGrapher_Sort.cgi?Bytes^$file_index^$ascend>       $totals_txt</a>";
        }
        if ($sampling_multiplier > 1) {
                $mbps_link   = "<a href=$cgi_bin_short/FlowGrapher_Sort.cgi?Mbps^$file_index^$ascend>   $ps_txt</a>";
        } else {
                $mbps_link   = "<a href=$cgi_bin_short/FlowGrapher_Sort.cgi?Mbps^$file_index^$ascend>    $ps_txt</a>";
        }
        print "$start_link$end_link$len_link$source_link$sport_link$dest_link$dport_link$bytes_link$mbps_link</b>\n\n";
        printf HTML "%-9s %-9s %6s %-20s  %-6s   %-20s  %-6s  %14s  %7s\n\n</b>", "Start", "End", "Len", "Source Host", "Port", "Destination Host", "Port", $totals_txt, $ps_txt;
}

# Output detail lines

for ($m=0;$m<=$detail_lines;$m++) {

	if ($sorted_lines[$m] eq "") { next; }

	($sort_field,$s_epoch,$s_tm,$e_tm,$len,$sip,$sp,$dip,$dp,$tb,$mbps) = split(/\s+/,$sorted_lines[$m]);

	if ($resolve_addresses eq "Y") {
		printf "%-9s %-9s %6.1f %-25.25s  %-6s   %-25.25s  %-6s  %14s  %8.3f\n", $s_tm, $e_tm, $len, $sip, $sp, $dip, $dp, $tb, $mbps;
		printf HTML "%-9s %-9s %6.1f %-25.25s  %-6s   %-25.25s  %-6s  %14s  %8.3f\n", $s_tm, $e_tm, $len, $sip, $sp, $dip, $dp, $tb, $mbps; }
	else {
		printf "%-9s %-9s %6.1f %-20.20s  %-6s   %-20.20s  %-6s  %14s  %8.3f\n", $s_tm, $e_tm, $len, $sip, $sp, $dip, $dp, $tb, $mbps;
		printf HTML "%-9s %-9s %6.1f %-20.20s  %-6s   %-20.20s  %-6s  %14s  %8.3f\n", $s_tm, $e_tm, $big_flow_length, $sip, $sp, $dip, $dp, $tb, $mbps;
	}
}

print "</pre>\n";
print HTML "</pre>\n";

print "<map name=FlowGrapherSave_Map>\n";  
print "<area shape=rect coords=\"0,0,75,88\" href=$cgi_bin_short/FlowGrapher.cgi?$call_string>\n";  
print "<area shape=rect coords=\"75,22,271,68\" href=$cgi_bin_short/FlowGrapher.cgi?$call_string>\n";  
print "<area shape=rect coords=\"75,0,271,22\" href=$cgi_bin_short/FlowViewer.cgi?$call_string>\n";  
print "<area shape=rect coords=\"75,68,271,88\" href=$cgi_bin_short/FlowTracker.cgi?$call_string>\n";  
print "<area shape=rect coords=\"275,0,350,30\" href=$cgi_bin_short/FlowViewer_Save.cgi?save_file^$save_filename>\n";   
print "<area shape=rect coords=\"275,50,350,80\" href=$cgi_bin_short/FlowViewer_Save.cgi?name_filter^$call_string>\n";
print "</map>\n";  
print HTML "<map name=FlowGrapher_Map>\n";  
print HTML "<area shape=rect coords=\"0,0,75,88\" href=$cgi_bin_short/FlowGrapher.cgi?$call_string>\n";  
print HTML "<area shape=rect coords=\"75,22,271,68\" href=$cgi_bin_short/FlowGrapher.cgi?$call_string>\n";  
print HTML "<area shape=rect coords=\"75,0,271,22\" href=$cgi_bin_short/FlowViewer.cgi?$call_string>\n";  
print HTML "<area shape=rect coords=\"75,68,271,88\" href=$cgi_bin_short/FlowTracker.cgi?$call_string>\n";  
print HTML "</map>\n";

print "</body></html>";
print HTML "</body></html>";
