#! /usr/bin/perl
#
#  Purpose:
#  FlowTracker.cgi creates the FlowTracker web page for inputting
#  selection criteria.
#
#  Description:
#
#  FlowTracker.cgi will create an form for accepting parameters to control
#  the filtering and selection of Netflow data in the generation of a
#  FlowTracker set of graphs.
#
#  Input arguments:
#  Name                 Description
#  -----------------------------------------------------------------------
#  none      
#
#  Modification history:
#  Author       Date            Vers.   Description
#  -----------------------------------------------------------------------
#  J. Loiacono  07/04/2006      3.0     Original version.
#  J. Loiacono  12/25/2006      3.1     Added listing of Archived trackings
#  J. Loiacono  02/14/2007      3.2     Changes to incoporate Groups
#  J. Loiacono  12/07/2007      3.3     Named Interfaces (thanks C. Kashimoto) 
#                                       Saved Filters, Next Hop
#
#$Author$
#$Date$
#$Header$
#
###########################################################################
#
#               BEGIN EXECUTABLE STATEMENTS
#
 
use FlowViewer_Configuration;
use FlowViewer_Utilities;

if ($debug_tracker eq "Y") { open (DEBUG,">$work_directory/DEBUG_TRACKER"); }

print "Content-type:text/html\n\n";
print "<html>\n";
print "<head>\n";
print "<title>\n";
print "FlowTracker $version\n";
print "</title>\n";
print "</head>\n";
print "<body bgcolor=$bg_color text=$text_color link=$link_color vlink=$vlink_color>\n";
print "<style type=text/css><!-- A { text-decoration:none } --></style>";
print "<center>\n";
print "<img src=$tracker_short/FlowTracker.png border=0 usemap=#FlowTracker_Map>"; 
if ($user_logo ne "") {  
        print "<a href=$user_hyperlink>"; 
        print "<img src=$reports_short/$user_logo border=0>"; 
        print "</a>";  
}     
print "</center><b><pre>";

# Create a Tracker directory if it doesn't exist

if (!-e $tracker_directory) { 

        mkdir($tracker_directory,$html_dir_perms) || die "cannot mkdir $tracker_directory: $!";   
        chmod $html_dir_perms, $tracker_directory;

	$group_directory = "$tracker_directory/GROUPS";
        mkdir($group_directory,$html_dir_perms) || die "cannot mkdir $group_directory: $!";   
        chmod $html_dir_perms, $group_directory;

	print "<pre><b>";
	print "\n";
        print "      The directory for storing Tracking files has been created:\n\n"; 
        print "      <i>$tracker_directory</i>\n\n"; 
        print "      Please ensure this directory has adequate permissions for your\n"; 
        print "      web server process owner (e.g., 'apache') to write into it.\n\n"; 
	print "<pre></b>";

	$new_directory = 1;
}

# Copy image into Tracker directory 
      
$copy_command = "cp $cgi_bin_directory/FlowTracker.png $tracker_directory"; 
system($copy_command); 

# Create directory to keep Tracker filter files, if it doesn't exist already
      
if (!-e $filter_directory) {  

        mkdir($filter_directory,$filter_dir_perms) || die "Cannot mkdir Tracker filter directory: $filter_directory: $!";  
        chmod $filter_dir_perms, $filter_directory;

	print "<pre><b>";
        print "      The directory for storing Tracking Filter files has been created:\n\n"; 
        print "      <i>$filter_directory</i>\n\n"; 
        print "      Please ensure this directory has adequate permissions for your\n"; 
        print "      web server process owner (e.g., 'apache') to write into it.\n\n"; 
	print "<pre></b>";

	$new_directory = 1;
} 

# Create directory to keep Tracker RRDtool files, if it doesn't exist already
      
if (!-e $rrdtool_directory) {  

        mkdir($rrdtool_directory,$rrd_dir_perms) || die "Cannot mkdir Tracker RRDtool directory: $rrdtool_directory: $!";  
        chmod $rrd_dir_perms, $rrdtool_directory;

	print "<pre><b>";
        print "      The directory for storing Tracking RRDtool files has been created:\n\n"; 
        print "      <i>$rrdtool_directory</i>\n\n"; 
        print "      Please ensure this directory has adequate permissions for your\n"; 
        print "      web server process owner (e.g., 'apache') to write into it.\n\n"; 
	print "</pre></b>";

	$new_directory = 1;
} 

if ($new_directory) {
	print "<pre><b>";
        print "      <a href=$cgi_bin_short/FlowTracker.cgi>RETURN</a>\n";
	print "</pre></b>";

	exit;
}

$query_string = $ENV{'QUERY_STRING'}; 
      
if ($query_string ne "") { 
        $query_string =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/ge ; 
        chop $query_string; 
        @fields = split(/&/,$query_string); 
        foreach $input_field (@fields) { 
                ($field,$field_value) = split(/=/,$input_field); 
                if ($field eq "filter_hash")         { $filter_hash = $field_value; }
                if ($field eq "device_name")         { $selected_device_name = $field_value; } 
                if ($field eq "start_date")          { $start_date = $field_value; } 
                if ($field eq "start_time")          { $start_time = $field_value; } 
                if ($field eq "end_date")            { $end_date = $field_value; } 
                if ($field eq "end_time")            { $end_time = $field_value; } 
                if ($field eq "source_addresses")    { $source_address = $field_value; } 
                if ($field eq "source_ports")        { $source_port = $field_value; } 
                if ($field eq "source_ifs")          { $source_if = $field_value; } 
                if ($field eq "sif_names")           { $selected_sif_name = $field_value; } 
                if ($field eq "source_ases")         { $source_as = $field_value; } 
                if ($field eq "dest_addresses")      { $dest_address = $field_value; } 
                if ($field eq "dest_ports")          { $dest_port = $field_value; } 
                if ($field eq "dest_ifs")            { $dest_if = $field_value; } 
                if ($field eq "dif_names")           { $selected_dif_name = $field_value; } 
                if ($field eq "dest_ases")           { $dest_as = $field_value; } 
                if ($field eq "protocols")           { $protocols = $field_value; } 
                if ($field eq "tos_fields")          { $tos_fields = $field_value; } 
                if ($field eq "tcp_flags")           { $tcp_flags = $field_value; } 
                if ($field eq "exporter")            { $selected_exporter = $field_value; } 
                if ($field eq "nexthop_ips")         { $nexthop_ip = $field_value; }
                if ($field eq "tracking_label")      { $tracking_label = $field_value; } 
                if ($field eq "general_comment")     { $general_comment = $field_value; } 
                if ($field eq "sampling_multiplier") { $sampling_multiplier = $field_value; } 
                if ($field eq "alert_threshold")     { $alert_threshold = $field_value; } 
                if ($field eq "alert_notification")  { $alert_notification = $field_value; } 
                if ($field eq "alert_destination")   { $alert_destination = $field_value; } 
        }        
}

if ($selected_device_name ne "") {
        $interfaces_file = "$cgi_bin_directory/NamedInterfaces_Devices";
} elsif ($selected_exporter ne "") {
        $interfaces_file = "$cgi_bin_directory/NamedInterfaces_Exporters";
}

# Create the header portion of the FlowTracker webpage
 
print "<form action=$cgi_bin_short/FlowTracker_Main.cgi method=POST>";
print "Filter Criteria:";

# Print out Saved Filters
 
$saved_filters_file = "$cgi_bin_directory/FlowViewer_SavedFilters";
if (-e $saved_filters_file) {
        print " <select name=node onchange=window.location.href=this[selectedIndex].value\; style=\"width: $pulldown_width\">\n";
        if ($filter_hash eq "") { print "<option selected>Select Saved Filter"; }
        open (FILTERS,"<$saved_filters_file");
        while (<FILTERS>) {
                chomp;
                ($existing_hash,$existing_name,$existing_string) = split(/\^/);
                $option_value = "$cgi_bin_short/FlowTracker.cgi?filter_hash=$existing_hash&$existing_string";
                if ($existing_hash eq $filter_hash) {
                        print "<option selected value=$option_value>$existing_name";
                        $current_hash = $existing_hash;
                } else {
                        print "<option value=$option_value>$existing_name";
                }
        }
        close FILTERS;
        $option_value = "$cgi_bin_short/FlowViewer_Save.cgi?name_filter^$current_hash";
        print "<option value=$option_value>Review or Modify Filters";
        print "</select>";
}
print "\n\n";

$pulldown_width = 140;
if ($devices[0] ne "") {
        print "         Device: <select name=device_name onchange=window.location.href=this[selectedIndex].value\; style=\"width: $pulldown_width\">\n";
        print "<option selected value=\"\">Select Device\n";
        foreach $device_name (@devices) {
                $option_value = "$cgi_bin_short/FlowTracker.cgi?device_name=$device_name&";
                if ($device_name eq $selected_device_name) {
                        print "<option selected value=$option_value>$device_name";
                } else {
                        print "<option value=$option_value>$device_name";
                }
        }
        print "</select>";
}
 
if ($exporters[0] ne "") {
        if ($devices[0] ne "") { print "&nbsp&nbsp&nbsp&nbsp&nbsp"; }
        print "       Exporter: <select name=exporter onchange=window.location.href=this[selectedIndex].value\; style=\"width: $pulldown_width\">\n";
        print "<option selected value=\"\">Select Exporter\n";
        foreach $exporter_pair (@exporters) {
                ($exporter_ip,$exporter_name) = split(/:/,$exporter_pair);
                $option_value = "$cgi_bin_short/FlowTracker.cgi?exporter=$exporter_ip&";
                if ($exporter_ip eq $selected_exporter) {
                        $exporter = $exporter_ip;
                        print "<option selected value=$option_value>$exporter_name";
                } else {
                        print "<option value=$option_value>$exporter_name";
                }
        }
        print "</select>";
}
 
if (($devices[0] ne "") && ($exporters[0] eq "")) {
        print "        Next Hop IP: <input type=text size=19 name=nexthop_ip value=\"$nexthop_ip\">"; }
if (($devices[0] eq "") && ($exporters[0] ne "")) {
        print "      Next Hop IP: <input type=text size=19 name=nexthop_ip value=\"$nexthop_ip\">"; }
if (($devices[0] ne "") && ($exporters[0] ne "")) {
        print "        Next Hop IP: <input type=text size=19 name=nexthop_ip value=\"$nexthop_ip\">"; }
if (($devices[0] eq "") && ($exporters[0] eq "")) {
        print "    Next Hop IP: <input type=text size=19 name=nexthop_ip value=\"$nexthop_ip\">"; }
print "\n\n";

print "       Protocol: <input type=text size=6 name=protocols value=$protocols>";
print "                    TOS Field: <input type=text size=10 name=tos_fields value=$tos_fields> </b>(e.g., -0x0b/0x0F)<b>";
print "       TCP Flag: <input type=text size=10 name=tcp_flags value=$tcp_flags> </b>(e.g., -0x0b/0x0F)<b>";
print "\n\n";

# Print out pulldown with named Source interfaces
 
print "      Source IP: <input type=text size=19 name=source_address value=\"$source_address\">";
print "    Source Interface: <input type=text size=2 name=source_if value=\"$source_if\">";
 
# Print out Include named interfaces
 
open (NAMED,"<$interfaces_file");
chomp (@interfaces = <NAMED>);
close (NAMED);

print "  <select name=sif_name width=20></b>\n";
print "<option selected value=\"\">Interface Names\n";
print "<optgroup label=\"Include ...\">";
foreach $interface (@interfaces) {
        if (($interface eq "") || (substr($interface,0,1) eq "#")) { next; }
        ($device,$interface_index,$interface_name) = split(/:/,$interface);
        if (($device eq $selected_device_name) || ($device eq $selected_exporter)) {
                if ($interface_index eq $selected_sif_name) {
                        print "<option selected value=$interface_index>$interface_index $interface_name\n";
                } else {
                        print "<option value=$interface_index>$interface_index $interface_name\n";
                }
        }
}
 
# Print out Exclude named interfaces
 
print "<optgroup label=\"Exclude ...\">";
foreach $interface (@interfaces) {
        if ($interface eq "") { next; }
        ($device,$interface_index,$interface_name) = split(/:/,$interface);
        $interface_index *= -1;
        if (($device eq $selected_device_name) || ($device eq $selected_exporter)) {
                if ($interface_index eq $selected_sif_name) {
                        print "<option selected value=$interface_index>$interface_index $interface_name\n";
                } else {
                        print "<option value=$interface_index>$interface_index $interface_name\n";
                }
        }
}
 
print "</select>";
print "       Source Port: <input type=text size=5 name=source_port value=\"$source_port\">";
print "   Source AS: <input type=text size=6 name=source_as value=\"$source_as\">\n\n";
 
# Print out pulldown with named Destination interfaces
 
print "        Dest IP: <input type=text size=19 name=dest_address value=\"$dest_address\">";
print "      Dest Interface: <input type=text size=2 name=dest_if value=\"$dest_if\">";
 
# Print out Include named interfaces
 
open (NAMED,"<$interfaces_file");
chomp (@interfaces = <NAMED>);
close (NAMED);

print "  <select name=dif_name width=20></b>\n";
print "<option selected value=\"\">Interface Names\n";
print "<optgroup label=\"Include ...\">";
foreach $interface (@interfaces) {
        if ($interface eq "") { next; }
        ($device,$interface_index,$interface_name) = split(/:/,$interface);
        if (($device eq $selected_device_name) || ($device eq $selected_exporter)) {
                if ($interface_index eq $selected_dif_name) {
                        print "<option selected value=$interface_index>$interface_index $interface_name\n";
                } else {
                        print "<option value=$interface_index>$interface_index $interface_name\n";
                }
        }
}
 
# Print out Exclude named interfaces
 
print "<optgroup label=\"Exclude ...\">";
foreach $interface (@interfaces) {
        if ($interface eq "") { next; }
        ($device,$interface_index,$interface_name) = split(/:/,$interface);
        $interface_index *= -1;
        if (($device eq $selected_device_name) || ($device eq $selected_exporter)) {
                if ($interface_index eq $selected_dif_name) {
                        print "<option selected value=$interface_index>$interface_index $interface_name\n";
                } else {
                        print "<option value=$interface_index>$interface_index $interface_name\n";
                }
        }
}
 
print "</select>";
print "         Dest Port: <input type=text size=5 name=dest_port value=\"$dest_port\">";
print "     Dest AS: <input type=text size=6 name=dest_as value=\"$dest_as\"></b>";

print "\n\n";
print "<center></b>Notes: Multiple field entries, separated by commas, are permitted in the fields above.\n";
print "A minus sign (-) will negate an entry (e.g. -1776 for AS, would mean any AS but 1776).\n";
print "IP fields accept networks (e.g. 192.168.10.0/19), hosts, and names (e.g. www.abc.com).<b></center>";
print "\n";
print "<hr size=4>";

# Output FlowTracker specific fields

print "Tracking Parameters:\n\n";
print "  Tracking Set Label: <input type=text size=40 name=tracking_label value=$tracking_label>";
print "    Tracking Type: <select name=tracking_type width=24></b>"; 
print "<option value=Individual selected>Individual\n"; 
print "<option value=Group>Group\n"; 
print "</select>";
print "    <b>Sampling Multiplier:</b> <input type=text size=3 name=sampling_multiplier value=$sampling_multiplier>";
print "\n\n";
print "     Alert Threshold: <input type=text size=12 name=alert_threshold value=$alert_threshold>";
print "     Alert Frequency: <select name=alert_frequency width=24>"; 
print "<option value=none selected>No Notification\n"; 
print "<option value=daily>Once a Day\n"; 
print "<option value=eachtime>Each Occurence\n"; 
print "</select>";
print "     Alert Destination: <input type=text size=22 name=alert_destination value=$alert_destination>";
print "\n\n";
print "     General Comment: <input type=text size=82 name=general_comment value=$general_comment>";
print "\n\n";
print "<center>";
print "<input type=hidden name=flow_select value=1>\n";
print "<input type=submit name=action value=\"Establish Tracking\">&nbsp\;<input type=reset value=\"Reset Values\">\n\n";
print "</center>";
print "<hr size=4>";

# List each existing tracking for revision, removal purposes

print "Individual Trackings:\n\n";

@filters = <$filter_directory/*>;
@filters = sort(@filters);

foreach $filter_file (@filters) {
	
        $tracking_file = $filter_file; 
        $tracking_file =~ s#.*/##; 

        open (FILTERS,"<$filter_file"); 
        while (<FILTERS>) { 
                chop;    
                $key = substr($_,0,8);
                if ($key eq " input: ") {

                        ($input,$field,$field_value) = split(/: /);

                        if ($field eq "tracking_label")   { 
				$filter_label = $field_value;

				$filter_file   = "$filter_directory/$tracking_file"; 
				($tracking_prefix,$tracking_suffix) = split(/\./,$tracking_file);
				$html_file     = "$tracker_short/$tracking_prefix/index.html";

				$file_label = $filter_label;
				$file_label =~ s/ /~/g;

				$filter_label_out = $filter_label;
				$length = length($filter_label_out);
	        		for ($i=$length;$i<=80;$i++) { $filter_label_out .= " "; }

				$tracking_link = "<a href=$html_file>$filter_label_out</a>";
				$revise_link   = "<a href=$cgi_bin_short/FlowTracker_Main.cgi?Solicit^$file_label>Revise</a>";
				$remove_link   = "<a href=$cgi_bin_short/FlowTracker_Main.cgi?Remove^$file_label>Remove</a>";

				if ($tracking_suffix eq "archive") {
					$num_archive++;
					$stop_link = "<a href=$cgi_bin_short/FlowTracker_Main.cgi?Restart^$file_label>Restart</a>";
					$revise_link = "";
					$archive_link = $tracking_link ."^^". $revise_link ."^^". $stop_link ."^^". $remove_link;
					$archive_links{$num_archive} = $archive_link;
					next;
				} elsif ($tracking_suffix eq "grp") {
					$num_group++;
					$revise_link   = "<a href=$cgi_bin_short/FlowTracker_Group.cgi?Solicit+$file_label>Revise</a>";
					$stop_link = "<a href=$cgi_bin_short/FlowTracker_Main.cgi?Archive^$file_label>Archive</a>"; 
					$group_link = $tracking_link ."^^". $revise_link ."^^". $stop_link ."^^". $remove_link;
					$group_links{$num_group} = $group_link;
					next;
				} else {
					$stop_link = "<a href=$cgi_bin_short/FlowTracker_Main.cgi?Archive^$file_label>Archive</a>"; 
				}

				printf "      %-56s %-8s    %-8s    %-8s\n", $tracking_link, $revise_link, $stop_link, $remove_link; }
			else {
				next;
			}
		}
	}
} 

if ($num_group > 0) {
	print "\nGroup Trackings:\n\n";
	@sorted_groups = sort { $a <=> $b} (keys %group_links);
	foreach $link_index (@sorted_groups) {
		($tracking_link,$revise_link,$stop_link,$remove_link) = split(/\^\^/,$group_links{$link_index});
		printf "      %-56s %-6s    %-8s    %-8s\n", $tracking_link, $revise_link, $stop_link, $remove_link;
	}
}

if ($num_archive > 0) {
	print "\nArchived Trackings:\n\n";
	@sorted_archives = sort { $a <=> $b } (keys %archive_links);
	foreach $link_index (@sorted_archives) {
		($tracking_link,$revise_link,$stop_link,$remove_link) = split(/\^\^/,$archive_links{$link_index});
		printf "      %-56s %-6s    %-8s    %-8s\n", $tracking_link, $revise_link, $stop_link, $remove_link;
	}
}

print "</pre>";
print "</form>\n";
print "<map name=FlowTracker_Map>\n";
print "<area shape=rect coords=\"0,0,75,88\" href=$cgi_bin_short/FlowTracker.cgi>\n";
print "<area shape=rect coords=\"75,22,271,68\" href=$cgi_bin_short/FlowTracker.cgi>\n";   
print "<area shape=rect coords=\"75,0,271,22\" href=$cgi_bin_short/FlowViewer.cgi>\n";   
print "<area shape=rect coords=\"75,68,271,88\" href=$cgi_bin_short/FlowGrapher.cgi>\n";   
print "</map>\n";
print "</body>\n";
print "</html>\n";
