#! /usr/bin/perl
#
#  Purpose:
#  FlowTracker_Dumper is invoked at the user's request to list out
#  the textual contents of the trackings.
#
#  Description:
#  A textual list of bits/second and byte quantities associated with the
#  relevant FlowTracking graph (i.e., Last 24 Hours, Last 7 Days, etc.)
#  is output to the user's web page. This routine simply invokes 
#  'rrdtool dump ...' to generate the text.
#
#  Controlling Parameters 
#  Name                 Description
#  -----------------------------------------------------------------------
#  tracking_name        Name of the tracking in question
#
#  Modification history:
#  Author       Date            Vers.   Description
#  -----------------------------------------------------------------------
#  J. Loiacono  12/07/2007      3.3     Original version.
#
#$Author$
#$Date$
#$Header$
#
###########################################################################
#
#               BEGIN EXECUTABLE STATEMENTS
#
 
use FlowViewer_Configuration;
use FlowViewer_Utilities;
use lib $cgi_bin_directory;

($dump_type,$tracking_label,$sampling_multiplier) = split(/\^/,$ENV{'QUERY_STRING'}); 

$tracking_label =~ s/~/ /g; 

# Retrieve current time to use as a file suffix to permit more than one user to list Tracking text
     
($sec,$min,$hr,$date,$mnth,$yr,$day,$yr_date,$DST) = localtime(time);
$mnth++; 
$yr += 1900; 
if ((0 < $mnth) && ($mnth < 10)) { $mnth = "0" . $mnth; }
if ((0 < $date) && ($date < 10)) { $date = "0" . $date; } 
if ((0 <= $hr)  && ($hr   < 10)) { $hr  = "0"  . $hr; } 
if ((0 <= $min) && ($min  < 10)) { $min = "0"  . $min; }  
if ((0 <= $sec) && ($sec  < 10)) { $sec = "0"  . $sec; }  
$prefix = $yr . $mnth . $date ."_". $hr . $min . $sec;  
$suffix = $hr . $min . $sec;

if ($dump_type eq "24_hours")  { $dump_type_out = "Last 24 Hours"; }
if ($dump_type eq "7_days")    { $dump_type_out = "Last 7 Days"; }
if ($dump_type eq "4_weeks")   { $dump_type_out = "Last 4 Weeks"; }
if ($dump_type eq "12_months") { $dump_type_out = "Last 12 Months"; }
if ($dump_type eq "3_years")   { $dump_type_out = "Last 3 Years"; }

$tracking_file = $tracking_label; 
$tracking_file =~ s/^\s+//; 
$tracking_file =~ s/\s+$//; 
$tracking_file =~ s/\&/-/g; 
$tracking_file =~ s/\//-/g; 
$tracking_file =~ s/\(/-/g; 
$tracking_file =~ s/\)/-/g; 
$tracking_file =~ s/\./-/g; 
$tracking_file =~ s/\s+/_/g; 
$tracking_file =~ tr/[A-Z]/[a-z]/;

$rrdtool_file  = "$rrdtool_directory/$tracking_file.rrd";
$dump_file     = "$work_directory/FlowTracker_dump_$suffix";

$dump_command = "$rrdtool_bin_directory/rrdtool dump $rrdtool_file > $dump_file";
system($dump_command);

# Get Averages

open (DUMP,"<$dump_file");
while (<DUMP>) {

	# Find start area

	if (($dump_type eq "24_hours")  && (/300 seconds/))   { $in_archive = 1; }
	if (($dump_type eq "7_days")    && (/1800 seconds/))  { $in_archive = 1; }
	if (($dump_type eq "4_weeks")   && (/7200 seconds/))  { $in_archive = 1; }
	if (($dump_type eq "12_months") && (/86400 seconds/)) { $in_archive = 1; }
	if (($dump_type eq "3_years")   && (/86400 seconds/)) { $in_archive = 1; }

	if (($in_archive)  && (/\<database/)) { $in_database = 1; };
	if (($in_database) && (/\/database/)) { $done = 1; };

	if ($in_database) { 

		($date,$time,$tz,$epoch,$value) = (split(/\s+/)) [2,3,4,6,9];
		($abs,$man) = split(/e\+/,$value);
		$value_int = int($abs * (10**$man));

		if ($dump_type eq "24_hours") { 
			$est_total_bytes = int($value_int * 300 / 8);
			$est_total_bytes = format_number($est_total_bytes);
		} elsif ($dump_type eq "7_days") { 
			$est_total_bytes = int($value_int * 1800 / 8);
			$est_total_bytes = format_number($est_total_bytes);
		} elsif ($dump_type eq "4_weeks") { 
			$est_total_bytes = int($value_int * 7200 / 8);
			$est_total_bytes = format_number($est_total_bytes);
		} elsif ($dump_type eq "12_months") { 
			$est_total_bytes = int($value_int * 86400 / 8);
			$est_total_bytes = format_number($est_total_bytes);
		} elsif ($dump_type eq "3_years") { 
			$est_total_bytes = int($value_int * 86400 / 8);
			$est_total_bytes = format_number($est_total_bytes);
		}

		$average = format_number($value_int);
		$line = join("^",$date,$time,$tz,$epoch,$average,$est_total_bytes);
		push (@averages,$line);
	}

	if ($done) { 
		$in_archive  = 0;
		$in_database = 0;
		$done        = 0;
		last; 
	}
}
close (DUMP);

# Get Maximums

open (DUMP,"<$dump_file");
while (<DUMP>) {

	# Find start area

	if (/MAX/) {$in_maximums = 1; }

	if (($in_maximums) && ($dump_type eq "24_hours")  && (/300 seconds/))   { $in_archive = 1; };
	if (($in_maximums) && ($dump_type eq "7_days")    && (/1800 seconds/))  { $in_archive = 1; };
	if (($in_maximums) && ($dump_type eq "4_weeks")   && (/7200 seconds/))  { $in_archive = 1; };
	if (($in_maximums) && ($dump_type eq "12_months") && (/86400 seconds/)) { $in_archive = 1; };
	if (($in_maximums) && ($dump_type eq "3_years")   && (/86400 seconds/)) { $in_archive = 1; };

	if (($in_archive)  &&  (/\<database/)) { $in_database = 1; };
	if (($in_database) &&  (/\/database/)) { $done = 1; };

	if ($in_database) { 
		($date,$time,$tz,$epoch,$value) = (split(/\s+/)) [2,3,4,6,9];
		($abs,$man) = split(/e\+/,$value);
		$value_int = int($abs * (10**$man));
		$maximum = format_number($value_int);
		$line = join("^",$date,$time,$tz,$epoch,$maximum);
		push (@maximums,$line);
	}

	if ($done) { last; }
}
close (DUMP);

# Output the report

print "Content-type:text/html\n\n"; 
print "<html>\n"; 
print "<head>\n"; 
print "<title>\n"; 
print "FlowTracker $version - Listing $dump_type_out for $tracking_label\n"; 
print "</title>\n"; 
print "</head>\n"; 
print "<body bgcolor=$bg_color text=$text_color link=$link_color vlink=$vlink_color>\n"; 
print "<style type=text/css><!-- A { text-decoration:none } --></style>";
print "<pre>\n";

print "<h3>Listing of the contents of the <i>\'$dump_type_out\'</i> for: <i>$tracking_label</i></h3>\n"; 

if ($sampling_multiplier ne "") { print "*Listed values are the original exported values multiplied by the Sampling Multiplier: $sampling_multiplier\n\n"; }

if ($dump_type eq "24_hours")  { print "Time represents end of 5-minute period (values are for the previous 5 minutes)      ";}
if ($dump_type eq "7_days")    { print "Time represents end of 30-minute period (values are for the previous 30 minutes)    ";}
if ($dump_type eq "4_weeks")   { print "Time represents end of 2-hour period (values are for the previous 2 hours)          ";}
if ($dump_type eq "12_months") { print "Time represents end of 24-hour period (values are for the previous day)             ";}
if ($dump_type eq "3_years")   { print "Time represents end of 24-hour period (values are for the previous day)             ";}
print "(nan = \'not a number\')\n\n";

if ($sampling_multiplier ne "") { 
	print "Date       Time     TZ    Epoch                Average(bps)*     Max 5-min (bps)*    Total Bytes (extrap.)*\n\n";
} else {
	print "Date       Time     TZ    Epoch                Average(bps)      Max 5-min (bps)     Total Bytes (extrap.)\n\n";
}

# Output each line

$num_records = $#averages;
for ($i=1;$i<$num_records;$i++) {
	($a_date,$a_time,$a_tz,$a_epoch,$average,$est_total_bytes) = split(/\^/,$averages[$i]);
	($m_date,$m_time,$m_tz,$m_epoch,$maximum)                  = split(/\^/,$maximums[$i]);
	printf "%-10s %-8s %-3s   %-10s   %20s %20s %25s\n",$a_date,$a_time,$a_tz,$a_epoch,$average,$maximum,$est_total_bytes;
}

# Remove the temporary dump file

$rm_command = "rm $dump_file";
system ($rm_command);
