#! /usr/bin/perl
#
#  Purpose:
#  FlowTracker_Group.cgi is invoked by FlowTracker_Main.cgi to
#  collect information from the user to build a Group Tracking.
#
#  Description:
#  The script responds to an HTML form from the user in order to collect
#  information taht will control the building of a group including,
#  component trackings, colors, and placements about teh x-axis.
#
#  Input arguments (received from the form):
#  Name                 Description
#  -----------------------------------------------------------------------
#  tracking_label       Label used for identifying Tracking filter and HTML file
#  general_comment      Useful for explaining a particular set of Tracking graphs
#
#  Modification history:
#  Author       Date            Vers.   Description
#  -----------------------------------------------------------------------
#  J. Loiacono  02/22/2007      3.2     Original version
#  J. Loiacono  12/07/2007      3.3     Fixed color processing, Logo links
#  J. Loiacono  04/15/2008      3.3.1   Fixed Groups missing x-axis (Wu Lei)
#
#$Author$
#$Date$
#$Header$
#
###########################################################################
#
#               BEGIN EXECUTABLE STATEMENTS
#
 
use FlowViewer_Configuration;
use FlowViewer_Utilities;
use RRDs;

if ($debug_group eq "Y") { open (DEBUG,">$work_directory/DEBUG_GROUP"); }

# Retrieve input parameters from CGI form invocation (POST)

read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'}); 
@pairs = split(/&/, $buffer); 
foreach $pair (@pairs) { 
    ($name, $value) = split(/=/, $pair);  
    $value =~ tr/+/ /; 
    $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C",hex($1))/eg; 
    $FORM{$name} = $value;  
} 

$action           = $FORM{action};
$tracking_label   = $FORM{tracking_label};
$general_comment  = $FORM{general_comment};
$revision_comment = $FORM{revision_comment};
$notate_graphs    = $FORM{notate_graphs};
$invoke_method    = "CGI";

# Retrieve input parameters from initial FlowTracker_Main.cgi invocation

if ($ARGV[0] ne "") {
	$invoke_method = "Inline";
	$tracking_info = $ARGV[0];
	$tracking_info =~ s/~/ /g;
	($action,$tracking_label,$general_comment) = split(/\+/,$tracking_info);
	if ($action eq "Solicit") { $action = "Solicit Revision"; }
}

if ($ARGV[1] ne "") { 
	$tracking_label = $ARGV[1]; 
	$tracking_label =~ s/\\//g;
	$tracking_label =~ s/~/ /g;
}

$tracking_file = $tracking_label;
$tracking_file =~ s/^\s+//;
$tracking_file =~ s/\s+$//;
$tracking_file =~ s/\&/-/g;
$tracking_file =~ s/\//-/g;
$tracking_file =~ s/\(/-/g;
$tracking_file =~ s/\)/-/g;
$tracking_file =~ s/\./-/g;
$tracking_file =~ s/\s+/_/g;
$tracking_file =~ tr/[A-Z]/[a-z]/;

$group_file = $filter_directory ."/". $tracking_file .".grp";

if ($debug_group eq "Y") {
	print DEBUG "In Flow_Tracker_Group.cgi ...\n";
	print DEBUG "  invoke method: $invoke_method\n";
	print DEBUG "        ARGV[0]: $ARGV[0]\n";
	print DEBUG "        ARGV[1]: $ARGV[1]\n";
	print DEBUG "         action: $action\n";
	print DEBUG "  tracking_info: $tracking_info\n";
	print DEBUG " tracking_label: $tracking_label\n";
	print DEBUG "  tracking_file: $tracking_file\n";
	print DEBUG "general_comment: $general_comment\n";
	print DEBUG "  tracking_file: $tracking_file\n";
}

# Create the GROUPS directory if it doesn't exist

$groups_directory = "$tracker_directory/GROUPS";
if (!-e $groups_directory) {
        mkdir $groups_directory, $html_dir_perms || die "cannot mkdir $groups_directory: $!";
        chmod $html_dir_perms, $groups_directory;
}

# Load the colors

$colors_file = "$cgi_bin_directory/FlowGrapher_Colors";

open (COLORS,"<$colors_file") || die "Can't open colors file; $colors_file\n";
while (<COLORS>) {
	chop;
	($red,$green,$blue,$color_1,$color_2) = split(/\s+/);
	$color_name = $color_1;
	if ($color_2 ne "") { $color_name = $color_1 . " " . $color_2; }
	$R = &dec2hex($red);   if (length($R) < 2) { $R = "0" . $R; }
	$G = &dec2hex($green); if (length($G) < 2) { $G = "0" . $G; }
	$B = &dec2hex($blue);  if (length($B) < 2) { $B = "0" . $B; }
	$hex_colors{$color_name} =  $R . $G . $B;
}
$hex_colors{"standard"} = $rrd_area;
sub dec2hex($) { return sprintf("%lx", $_[0]) }

# Retrieve existing group information for revision

if ($action eq "Solicit Revision") {

	open (GROUP,"<$group_file");
	while (<GROUP>) {
	        chop;
	        $key = substr($_,0,8);
	        if ($key eq " input: ") {
	                ($input,$field,$field_value) = split(/: /);
	                if ($field eq "general_comment")  { $general_comment = $field_value; }
		}
	}
	close (GROUP);
}

# Revise Group Comments

if ($action eq "Revise Group Comments") {

	open (GROUP,"<$group_file");
	while (<GROUP>) {
		$next_line = $_;
	        $key = substr($_,0,8);
	        if ($key eq " input: ") {
	                ($input,$field,$field_value) = split(/: /);
	                if ($field eq "general_comment")  { 
			$next_line = " input: general_comment: $general_comment\n"; }
	                if ($field eq "revision")  { $revision_num++; }
		}
		push (@group_lines,$next_line);
	}
	close (GROUP);

	open (GROUP,">$group_file");
	foreach $input_line (@group_lines) { print GROUP $input_line; }
	close (GROUP);

	if (($revision_comment ne "") && ($revision_num < 3)) {
		open (GROUP,">>$group_file");
		$revision_date = time - 60;
        	print GROUP " input: revision: $notate_graphs|$revision_date|$revision_comment\n";
		close (GROUP);
	}

	chmod $filter_file_perms, $group_file;
}

# Add a new component

if ($action eq "Add this Component") {

	$add_label    = $FORM{add_label};
	$add_location = $FORM{add_location};
	$add_color    = $FORM{add_color};

	open (GROUP,"<$group_file");
	@group_lines = <GROUP>;
	close (GROUP);

	foreach $group_line (@group_lines) {
		if ($group_line =~ / input:/) { push (@input_lines,$group_line); }
		else { push (@components,$group_line); }
	}

	$largest_above = 100;
	$largest_below = 200;

	foreach $component (@components) {

		($component_position,$component_label,$component_color) = split(/\^/,$component);

		if (($component_position<200) && ($component_position>$largest_above)) { $largest_above = $component_position; }
		if (($component_position>200) && ($component_position>$largest_below)) { $largest_below = $component_position; }

		if ($component_color =~ /auto blue/) { 
			$next_blue = substr($component_color,9,1);
			if ($component_position < 200) {
				if ($next_blue > $largest_blue_above) { $largest_blue_above = $next_blue; }
			}
			if ($component_position > 200) {
				if ($next_blue > $largest_blue_below) { $largest_blue_below = $next_blue; }
			}
		} 
		if ($component_color =~ /auto red/) { 
			$next_red = substr($component_color,8,1);
			if ($component_position < 200) {
				if ($next_red > $largest_red_above) { $largest_red_above = $next_red; }
			}
			if ($component_position > 200) {
				if ($next_red > $largest_red_below) { $largest_red_below = $next_red; }
			}
		} 
		if ($component_color =~ /auto violet/) { 
			$next_violet = substr($component_color,11,1);
			if ($component_position < 200) {
				if ($next_violet > $largest_violet_above) { $largest_violet_above = $next_violet; }
			}
			if ($component_position > 200) {
				if ($next_violet > $largest_violet_below) { $largest_violet_below = $next_violet; }
			}
		} 
		if ($component_color =~ /auto green/) { 
			$next_green = substr($component_color,10,1);
			if ($component_position < 200) {
				if ($next_green > $largest_green_above) { $largest_green_above = $next_green; }
			}
			if ($component_position > 200) {
				if ($next_green > $largest_green_below) { $largest_green_below = $next_green; }
			}
		} 
	}

	$add_color_value = $add_color;
	$add_color =~ s/_/ /;
	if (($add_location eq "above") && ($add_color eq "auto blue"))  { $add_color .= $largest_blue_above + 1; }
	if (($add_location eq "above") && ($add_color eq "auto red"))   { $add_color .= $largest_red_above + 1; }
	if (($add_location eq "above") && ($add_color eq "auto violet")){ $add_color .= $largest_violet_above + 1; }
	if (($add_location eq "above") && ($add_color eq "auto green")) { $add_color .= $largest_green_above + 1; }

	if (($add_location eq "below") && ($add_color eq "auto blue"))  { $add_color .= $largest_blue_below + 1; }
	if (($add_location eq "below") && ($add_color eq "auto red"))   { $add_color .= $largest_red_below + 1; }
	if (($add_location eq "below") && ($add_color eq "auto violet")){ $add_color .= $largest_violet_below + 1; }
	if (($add_location eq "below") && ($add_color eq "auto green")) { $add_color .= $largest_green_below + 1; }

	$add_component = $add_label ."^". $add_color ."\n";;

	# Determine new position

	if (($add_location eq "above") && ($largest_above == 100)) {
		$add_component = $largest_above+1 ."^". $add_component;
		push (@new_group,$add_component);
	}

	foreach $component (@components) {

		($component_position,$component_label,$component_color) = split(/\^/,$component);
		push (@new_group,$component);
		if (($add_location eq "above") && ($component_position == $largest_above)) {
			$add_component = $largest_above+1 ."^". $add_component;
			push (@new_group,$add_component);
		}
		if (($add_location eq "below") && ($component_position == $largest_below)) {
			$add_component = $largest_below+1 ."^". $add_component;
			push (@new_group,$add_component);
		}
	}

	if (($add_location eq "below") && ($largest_below == 200)) {
		$add_component = $largest_below+1 ."^". $add_component;
		push (@new_group,$add_component);
	}

	# Rewrite group file

	open (GROUP,">$group_file");
	foreach $component (@new_group)    { print GROUP $component; }
	foreach $input_line (@input_lines) { print GROUP $input_line; }
	close (GROUP);
	chmod $filter_file_perms, $group_file;
}

# Adjust the group

if ($action eq "Adjust the Group") {

	open (GROUP,"<$group_file");
	@group_lines = <GROUP>;
	close (GROUP);

	foreach $group_line (@group_lines) {
		if ($group_line =~ / input:/) { 
			push (@input_lines,$group_line);
		} else { 
			push (@components,$group_line); 

			($component_position,$component_label,$component_color) = split(/\^/,$group_line);

			if ($component_color =~ /auto blue/) { 
				$next_blue = substr($component_color,9,1);
				if ($component_position < 200) {
					if ($next_blue > $largest_blue_above) { $largest_blue_above = $next_blue; }
				}
				if ($component_position > 200) {
					if ($next_blue > $largest_blue_below) { $largest_blue_below = $next_blue; }
				}
			} 
			if ($component_color =~ /auto red/) { 
				$next_red = substr($component_color,8,1);
				if ($component_position < 200) {
					if ($next_red > $largest_red_above) { $largest_red_above = $next_red; }
				}
				if ($component_position > 200) {
					if ($next_red > $largest_red_below) { $largest_red_below = $next_red; }
				}
			} 
			if ($component_color =~ /auto violet/) { 
				$next_violet = substr($component_color,11,1);
				if ($component_position < 200) {
					if ($next_violet > $largest_violet_above) { $largest_violet_above = $next_violet; }
				}
				if ($component_position > 200) {
					if ($next_violet > $largest_violet_below) { $largest_violet_below = $next_violet; }
				}
			} 
			if ($component_color =~ /auto green/) { 
				$next_green = substr($component_color,10,1);
				if ($component_position < 200) {
					if ($next_green > $largest_green_above) { $largest_green_above = $next_green; }
				}
				if ($component_position > 200) {
					if ($next_green > $largest_green_below) { $largest_green_below = $next_green; }
				}
			} 
		}
	}

	$largest_red_above++;
	$largest_blue_above++;
	$largest_green_above++;
	$largest_violet_above++;
	$largest_red_below++;
	$largest_blue_below++;
	$largest_green_below++;
	$largest_violet_below++;

	foreach $component (@components) {
		$num_components++;
		chop $component;	
		($component_position,$component_label,$component_color) = split(/\^/,$component);

		$new_color = "new_color_" . $component_position;
		$component_color = $FORM{$new_color};
		$component_color =~ s/_/ /;
		
		if ($component_position < 200) {
			if ($component_color eq "auto red") { $component_color = "auto_red" . $largest_red_above; }
			if ($component_color eq "auto blue") { $component_color = "auto_blue" . $largest_blue_above; }
			if ($component_color eq "auto green") { $component_color = "auto_green" . $largest_green_above; }
			if ($component_color eq "auto violet") { $component_color = "auto_violet" . $largest_violet_above; }
		} else {
			if ($component_color eq "auto red") { $component_color = "auto_red" . $largest_red_below; }
			if ($component_color eq "auto blue") { $component_color = "auto_blue" . $largest_blue_below; }
			if ($component_color eq "auto green") { $component_color = "auto_green" . $largest_green_below; }
			if ($component_color eq "auto violet") { $component_color = "auto_violet" . $largest_violet_below; }
		}

		$component = $component_position ."^". $component_label ."^". $component_color;
		push (@adjusted_components,$component);

		$adjust_component_color = $component_color;
		$adjust_component_color =~ s/_/ /;
		$component_data =  $component_label ."^". $adjust_component_color;
		$placements{$component_position} = $component_data;

		if (($component_position<200) && ($component_position>$largest_above)) { $largest_above = $component_position; }
		if (($component_position>200) && ($component_position>$largest_below)) { $largest_below = $component_position; }
	}

	foreach $position (keys (%placements)) {
		($component_label,$component_color) = split(/\^/,$placements{$position});
		$new_position = "new_position_" . $position;
		$move_component = $FORM{$new_position};
		if ($move_component eq "move_away") {
			if (($position == $largest_above) || ($position == $largest_below)) { next; }
			$temp_component = $placements{$position+1};
			$placements{$position+1} = $placements{$position};
			$placements{$position} = $temp_component;
		} elsif ($move_component eq "move_closer") {
			if (($position == 101) || ($position == 201)) { next; }
			$temp_component = $placements{$position-1};
			$placements{$position-1} = $placements{$position};
			$placements{$position} = $temp_component;
		} elsif ($move_component eq "remove") { 
			if ($position < 200) {
				if ($position == $largest_above) { 
					delete $placements{$position};
					$largest_above--;
					next;
				}
				for ($i=$position+1;$i<=$largest_above;$i++) { $placements{$i-1} = $placements{$i}; }
				delete $placements{$largest_above};
				$largest_above--;
			}
			if ($position > 200) {
				if ($position == $largest_below) { 
					delete $placements{$position};
					$largest_below--;
					next;
				}
				for ($i=$position+1;$i<=$largest_below;$i++) { $placements{$i-1} = $placements{$i}; }
				delete $placements{$largest_below};
				$largest_below--;
			}
		}
	}

	@sorted_keys = sort keys(%placements);
	foreach $position (@sorted_keys) {
		$component = $position ."^". $placements{$position};
		push (@sorted_components,$component);
	}

	open (GROUP,">$group_file");
	foreach $component (@sorted_components) { print GROUP "$component\n"; }
	foreach $input_line (@input_lines) { print GROUP $input_line; }
	close (GROUP);
	chmod $filter_file_perms, $group_file;
}

print "Content-type:text/html\n\n";
print "<head>";
print "<style type=text/css><!-- A { text-decoration:none } --></style>";
print "</head>";
print "<body bgcolor=$bg_color>";
print "<title>FlowTracker $version Group Tracking</title>";
print "<pre>\n";
print "<b>";
print "<center>";
print "<table><tr height=100>\n";
print "<td>";
print "<a href=$cgi_bin_short/FlowTracker.cgi><img src=$tracker_short/FlowTracker.png border=0 usemap=#FlowTracker_Map></a>";
if ($user_logo ne "") {    
        print "<a href=$user_hyperlink>";  
        print "<img src=$reports_short/$user_logo border=0>";  
        print "</a>";   
}      
print "</td><td>";
print "<font face=arial size=+2>$tracking_label</font>\n"; 
print "</td>";
print "</tr></table>";

$end_rrd   = time;
$start_rrd = $end_rrd - 86400; 

# Gather existing components (if any) of this group

$num_components = 0;
$first_below = 1;
open (GROUP,"<$group_file") || die "Can't open group file: $group_file";
while (<GROUP>) {
	$num_components++;
	chop;	

	if (/ input:/) { 
		$num_components--;
        	($input,$field,$field_value) = split(/: /);
                if ($field eq "revision") {  
     
                        ($notate_graphs,$revision_date,$revision_comment) = split(/\|/,$field_value);   
                        $revision_date_out = epoch_to_date($revision_date,"LOCAL");  
                        $revision_date_out =~ s/:/\\:/g; 
     
                        if ($notate_graphs eq "Y") {    
                                if ($vrule_1 eq "") {  
					$revision_date = $start_rrd + 21600;
                                        $vrule_1 = " VRULE:$revision_date#$rrd_vrule_color:\"$revision_date_out\\: $revision_comment\\n\"";  
                                        next; }   
                                elsif ($vrule_2 eq "") {  
					$revision_date = $start_rrd + 43200;
                                        $vrule_2 = " VRULE:$revision_date#$rrd_vrule_color:\"$revision_date_out\\: $revision_comment\\n\"";  
                                        next; }   
                                elsif ($vrule_3 eq "") {  
					$revision_date = $start_rrd + 64800;
                                        $vrule_3 = " VRULE:$revision_date#$rrd_vrule_color:\"$revision_date_out\\: $revision_comment\\n\"";  
                                        next; }   
                        }        
		}
		next;
	}

	($component_position,$component_label,$component_color) = split(/\^/,$_);

	$component_rrd = "$work_directory/COMPONENT_SAMPLE$num_components.rrd";

	$DEF_parameters  .= "DEF:flowbits$num_components=$component_rrd:flowbits:AVERAGE ";
        $AREA_parameters .= "COMMENT:\"     \" ";

	if ($component_position < 200) {
		$AREA_parameters .= "AREA:flowbits$num_components#$hex_colors{$component_color}:\"$component_label\\n\":STACK ";
	} elsif (($component_position >= 200) && ($first_below)) {
		$DEF_parameters .= "CDEF:flowbits_below$num_components=flowbits$num_components,-1,* ";
		$AREA_parameters .= "AREA:flowbits_below$num_components#$hex_colors{$component_color}:\"$component_label\\n\" ";
		$first_below = 0;
	} else {
		$DEF_parameters .= "CDEF:flowbits_below$num_components=flowbits$num_components,-1,* ";
		$AREA_parameters .= "AREA:flowbits_below$num_components#$hex_colors{$component_color}:\"$component_label\\n\":STACK ";
	}
}

# Generate Sample Graph from Sample RRDtool database

$rm_command = "rm -rf $tracker_directory/GROUPS/*";
system($rm_command);

$suffix = rand();
$suffix = int ($suffix * 100);
$graph_file      = "$tracker_directory/GROUPS/GROUP_SAMPLE$suffix.png";
$graph_file_link = "$tracker_short/GROUPS/GROUP_SAMPLE$suffix.png";

if ($num_components == 0) {

	$rrdtool_file    = "$work_directory/COMPONENT_SAMPLE.rrd";
	$rrdtool_command = "$rrdtool_bin_directory/rrdtool create $rrdtool_file ".  
			   "--step 300 ". 
	                   "--start $start_rrd ". 
	                   "DS:flowbits:GAUGE:600:U:U ".  
	                   "RRA:AVERAGE:0.5:1:600 ";  

	system($rrdtool_command);  
	chmod $rrd_file_perms, $rrdtool_file; 

	$DEF_parameters  .= "DEF:flowbits0$num_components=$rrdtool_file:flowbits:AVERAGE ";
        $AREA_parameters .= "COMMENT:\"     \" ";
	$AREA_parameters .= "AREA:flowbits0$num_components#FFFFFF::STACK ";
}

for ($i=1;$i<=$num_components;$i++) {

	$rrdtool_file    = "$work_directory/COMPONENT_SAMPLE$i.rrd";
	$rrdtool_command = "$rrdtool_bin_directory/rrdtool create $rrdtool_file ".  
			   "--step 300 ". 
	                   "--start $start_rrd ". 
	                   "DS:flowbits:GAUGE:600:U:U ".  
	                   "RRA:AVERAGE:0.5:1:600 ";  

	system($rrdtool_command);  
	chmod $rrd_file_perms, $rrdtool_file; 

	$num_point = 0;
	for ($j=1;$j<=4;$j++) { 
	        for ($k=-36;$k<36;$k++) { 
	                $num_point++;  
	                $next_time = $start_rrd + ($num_point * 300);  
	                $x = ($k / 36) * 3.14159; 
			$sinx = sin ($x);
	                $next_value = int(300000 + ($sinx * 30000)); 
			RRDs::update ("$rrdtool_file","$next_time:$next_value");
			$ERR=RRDs::error;
			if (($debug_group eq "Y") && ($ERR ne "")) { print DEBUG "RRDs ERR: $ERR\n"; }
	        }        
	} 
}

$tracking_label_out = $tracking_label;
$tracking_label =~ s/\s+$//;
$tracking_label_out =~ s/\s+/_/g;
$rrd_title = $tracking_label_out . "_Sample";

@graph_parameters = 
        ('--title',"$rrd_title", 
        '--start',$start_rrd, 
        '--end',$end_rrd, 
        '--width',$rrd_width, 
        '--height',$rrd_height, 
        '--interlace', 
        '--vertical-label',"\"Bits per Second\"", 
        $rrd_slope_mode, 
        "--color=FONT#$rrd_font", 
        "--color=BACK#$rrd_back", 
        "--color=CANVAS#$rrd_canvas", 
        "--color=GRID#$rrd_grid", 
        "--color=MGRID#$rrd_mgrid", 
        "--color=FRAME#$rrd_frame", 
        "--color=SHADEA#$rrd_frame", 
        "--color=SHADEB#$rrd_frame", 
        '--lower-limit',$rrd_lower_limit, 
	$DEF_parameters,
        "COMMENT:\"\\n\"", 
	$AREA_parameters,
        "COMMENT:\"\\n\"", 
        "COMMENT:\"     \"", 
        $vrule_1, 
        "COMMENT:\"     \"", 
        $vrule_2, 
        "COMMENT:\"     \"", 
        $vrule_3);

$rrdgraph_command = "$rrdtool_bin_directory/rrdtool graph " . "$graph_file " . "@graph_parameters " . ">/dev/null";
system($rrdgraph_command);
chmod $tracker_file_perms, $graph_file;

print "  <img src=$graph_file_link>\n";
print "</center>";
print "<b><pre>\n"; 
print "<hr size=4>"; 

# Clean up temporary COMPONENT RRD files

$cleanup_command = "rm $work_directory/COMPONENT_SAMPLE*";
if ($debug_files ne "Y") { system($cleanup_command); }

# Build list of existing trackings to select from

print " Select an existing Tracking to be a component of this Group:\n\n\n";
print "<form action=$cgi_bin_short/FlowTracker_Group.cgi method=POST>";
print "         Select a Component: <select name=add_label length=15></b>\n";  

print "Building trackings\n";

while ($filter_file = <$filter_directory/*>) { 
        ($directory,$tracking) = $filter_file =~ m/(.*\/)(.*)$/; 
        ($tracking,$extension) = split(/\./,$tracking); 

	if ($extension eq "fil") {
		$grep_command = "grep tracking_label $filter_file";
		open(GREP,"$grep_command 2>&1|"); 
		while (<GREP>) {
			chop;
			($input,$label,$tracking) = split(/:/);
			$tracking = substr($tracking,1,100);
			print "    <option value=\"$tracking\">$tracking\n";  
		}
	}
}
print "    </select>\n\n"; 

print "       Place this Component: <select name=add_location length=15></b>\n";  
print "    <option selected value=above>Above the X-axis\n";  
print "    <option value=below>Below the X-axis\n";  
print "    </select>\n\n"; 

open (COLORS,"<$cgi_bin_directory/FlowGrapher_Colors");
print "             Select a Color: <select name=add_color length=15></b>\n";  

if (substr($add_color,0,8) eq "auto red") { 
	print "<option selected value=auto_red>Auto Red\n";  
	print "<option value=auto_blue>Auto Blue\n";  
	print "<option value=auto_green>Auto Green\n";  
	print "<option value=auto_violet>Auto Violet\n";  
} elsif (substr($add_color,0,10) eq "auto green") { 
	print "<option selected value=auto_green>Auto Green\n";  
	print "<option value=auto_red>Auto Red\n";  
	print "<option value=auto_blue>Auto Blue\n";  
	print "<option value=auto_violet>Auto Violet\n";  
} elsif (substr($add_color,0,11) eq "auto violet") { 
	print "<option selected value=auto_violet>Auto Violet\n";  
	print "<option value=auto_red>Auto Red\n";  
	print "<option value=auto_blue>Auto Blue\n";  
	print "<option value=auto_green>Auto Green\n";  
} else {
	print "<option selected value=auto_blue>Auto Blue\n";  
	print "<option value=auto_green>Auto Green\n";  
	print "<option value=auto_violet>Auto Violet\n";  
	print "<option value=auto_red>Auto Red\n";  
}

print "<option value=standard>standard\n";  
while (<COLORS>) {
	($r,$g,$b,$color1,$color2) = split(/\s+/,$_);
	$next_color = "$color1";
	$next_value = "$color1";
	if ($color2 ne "") { $next_color = $color1 ." ". $color2; }
	if ($color2 ne "") { $next_value = $color1 ."_". $color2; }
	$color_choice=  "    <option value=$next_value>$next_color\n";  
	print $color_choice;
	push(@color_choices,$color_choice);
}
print "    </select>\n\n\n"; 

print "<center>"; 
print "<input type=hidden name=tracking_label value=\"$tracking_label\">\n"; 
print "<input type=submit name=action value=\"Add this Component\">&nbsp\;<input type=reset value=\"Reset Values\">\n\n"; 
print "</center>"; 
print "<hr size=4>"; 

# List the components so that a user can modify them, their position, their order, or their color

print " This group is composed of these components:\n\n\n";
print "<form action=$cgi_bin_short/FlowTracker_Group.cgi method=POST>";

open (GROUP,"<$group_file");
@components = <GROUP>;
close (GROUP);

$num_components = 0;
foreach $component (@components) {

	$num_components++;

	if ($component =~ / input:/) { $num_components--; next; }

	chop $component;	
	($component_position,$component_label,$component_color) = split(/\^/,$component);

	$component_parameter = $component_label;
	$component_parameter =~ s//~/g;

	$component_label_out = $component_label;
	$len_component = length($component_label);
	if ($len_component > 40) { 
		$start_char = $len_component - 38;
		$component_label_out = substr($component_label_out,$start_char,38);
		$component_label_out = "..." . $component_label_out;
	}
	for ($i=$len_component;$i<=40;$i++) { $component_label_out .= " "; }
	$component_link = "<a href=$cgi_bin_short/FlowTracker_Main.cgi?Solicit^$component_parameter>$component_label_out</a>";

	$component_color_value = $component_color;
	$component_color_value =~ s/ /_/;
	$component_color_out = $component_color;
	$len_color = length($component_color);
	for ($i=$len_color;$i<=16;$i++) { $component_color_out .= " "; }
	$color_link = "<font color=$hex_colors{$component_color}>$component_color_out</font>";

	$axis_position  = substr($component_position,0,1);
	$stack_position = substr($component_position,1,2);
	if ($axis_position == 1) {
		$component_position_out = "Above ";
	} else {
		$component_position_out = "Below ";
	}
	$component_position_out .= $stack_position;

	$next_component = $component_position ."##". $component_position_out ."##". $component_link ."##". $color_link ."##". $component_color ."##". $component_color_value;
	if ($component_position < 200) { push (@upper_components,$next_component); }
	if ($component_position > 200) { push (@lower_components,$next_component); }
}

# Print out components that appear ABOVE the axis

@reversed_uppers = reverse (@upper_components);
foreach $upper_component (@reversed_uppers) {

	($component_position,$component_position_out,$component_link,$color_link,$component_color,$component_color_value) = split(/##/,$upper_component);

	printf " %-40s %-8s  %-16s", $component_link, $component_position_out, $color_link;

	$new_color = "new_color_" . $component_position;
	print "New Color: <select name=$new_color length=15></b>\n";  
	print "<option selected value=$component_color_value>$component_color\n";  
	print "<option value=standard>standard\n";  
	print "<option value=auto_blue>Auto Blue\n";  
	print "<option value=auto_red>Auto Red\n";  
	print "<option value=auto_green>Auto Green\n";  
	print "<option value=auto_violet>Auto Violet\n";  
	foreach $color_choice (@color_choices) { print $color_choice; }
	print "</select>"; 

	$new_position = "new_position_" . $component_position;
	print "  Move: <select name=$new_position length=15></b>\n";  
	print "<option selected value=leave_alone>Leave Alone\n";  
	print "<option value=move_away>Move Away from Axis\n";  
	print "<option value=move_closer>Move Closer to Axis\n";  
	print "<option value=remove>Remove\n";  
	print "</select>\n"; 
}

# Print out components that appear BELOW the axis

if (@lower_components) { print "<hr size=1 width=600>"; } 

foreach $lower_component (@lower_components) {

	($component_position,$component_position_out,$component_link,$color_link,$component_color,$component_color_value) = split(/##/,$lower_component);

	printf " %-40s %-8s  %-16s", $component_link, $component_position_out, $color_link;

	$new_color = "new_color_" . $component_position;
	print "New Color: <select name=$new_color length=15></b>\n";  
	print "<option selected value=$component_color_value>$component_color\n";  
	print "<option value=standard>standard\n";  
	print "<option value=auto_blue>Auto Blue\n";  
	print "<option value=auto_red>Auto Red\n";  
	print "<option value=auto_green>Auto Green\n";  
	print "<option value=auto_violet>Auto Violet\n";  
	foreach $color_choice (@color_choices) { print $color_choice; }
	print "</select>"; 

	$new_position = "new_position_" . $component_position;
	print "  Move: <select name=$new_position length=15></b>\n";  
	print "<option selected value=leave_alone>Leave Alone\n";  
	print "<option value=move_away>Move Away from Axis\n";  
	print "<option value=move_closer>Move Closer to Axis\n";  
	print "<option value=remove>Remove\n";  
	print "</select>\n"; 
}

print "<br>";
print "<center>"; 
print "<input type=hidden name=tracking_label value=\"$tracking_label\">\n"; 
print "<input type=submit name=action value=\"Adjust the Group\">&nbsp\;<input type=reset value=\"Reset Values\">\n\n"; 
print "</center>"; 
print "<hr size=4>"; 

# List out current Comment and Revision input

print "<form action=$cgi_bin_short/FlowTracker_Group.cgi method=POST>";
print " Change Comment or add a Revision Mark (Maximum three revisions):\n\n\n";
print "     General Comment: <input type=text size=90 name=general_comment value=\"$general_comment\">"; 
print "\n\n";  
$revision_comment = "";
print "    Revision Comment: <input type=text size=90 name=revision_comment value=\"$revision_comment\">"; 
print "\n\n";  
print "       Notate Graphs: <select name=notate_graphs width=4></b>\n";   
print "    <option selected value=N>N\n";
print "    <option value=Y>Y\n";
print "    </select></b> (To notate graphs with time and description of revision)";
print "\n\n";
print "<center>";
print "<input type=hidden name=tracking_label value=\"$tracking_label\">\n"; 
print "<input type=submit name=action value=\"Revise Group Comments\">&nbsp\;<input type=reset value=\"Reset Values\">\n\n"; 
print "</form>\n"; 
print "<hr size=4>"; 

# Print Done button

print "\n";
$done_link = "$FlowViewer_service://$FlowViewer_server$cgi_bin_short/FlowTracker.cgi";
print "<form action=$done_link method=link><input type=submit value=Done></form>";

print "\n\n";
print "</center>";
print "</pre>"; 
print "<map name=FlowTracker_Map>\n"; 
print "<area shape=rect coords=\"0,0,75,88\" href=$cgi_bin_short/FlowTracker.cgi?$call_string>\n"; 
print "<area shape=rect coords=\"75,22,271,68\" href=$cgi_bin_short/FlowTracker.cgi?$call_string>\n"; 
print "<area shape=rect coords=\"75,0,271,22\" href=$cgi_bin_short/FlowViewer.cgi?$call_string>\n"; 
print "<area shape=rect coords=\"75,68,271,88\" href=$cgi_bin_short/FlowGrapher.cgi?$call_string>\n"; 
print "</map>\n"; 
print "</body>\n"; 
print "</html>\n"; 

sub print_error { 

        my ($error_text) = @_; 

	$rm_command = "rm $filter_file";
	system($rm_command);

        print "<head>"; 
        print "</head>"; 
        print "<body bgcolor=$bg_color>"; 
        print "<title>FlowTracker Error</title>"; 
        print "<center>\n"; 
        print "<h3>$error_text</h3><br>"; 
	print " Use browser \"bask\" key to preserve inputs for modification<br>";
        print "<br>\n"; 
        print "<form action=$FlowViewer_service://$FlowViewer_server$cgi_bin_short/FlowTracker.cgi method=link>"; 
        print "<input type=submit value=\"New FlowTracker Inputs\"></form>"; 
     
        exit;    
}    
