#! /usr/bin/perl
#
#  Purpose:
#  FlowViewer.cgi creates the FlowViewer web page for inputting
#  selection criteria.
#
#  Description:
#
#  FlowViewer.cgi will create an form for accepting parameters
#  to control the filtering and selection of Netflow data.
#
#  Input arguments:
#  Name                 Description
#  -----------------------------------------------------------------------
#  none      
#
#  Modification history:
#  Author       Date            Vers.   Description
#  -----------------------------------------------------------------------
#  J. Loiacono  07/04/2005      1.0     Original version.
#  J. Loiacono  01/01/2006      2.0     New filter parameters
#  J. Loiacono  01/26/2006      2.2     Added flow_select option
#  J. Loiacono  07/04/2006      3.0     Replaces create_FlowViewer_webpage
#  J. Loiacono  12/25/2006      3.1     [No Change to this module]
#  J. Loiacono  02/22/2007      3.2     [No Change to this module]
#  J. Loiacono  12/07/2007      3.3     Named Interfaces, Octets Conv. 
#                                       (thanks C. Kashimoto)
#                                       Saved Filters, Next Hop
#
#$Author$
#$Date$
#$Header$
#
###########################################################################
#
#               BEGIN EXECUTABLE STATEMENTS
#
 
use FlowViewer_Configuration;
use FlowViewer_Utilities;

if ($debug_viewer eq "Y") { open (DEBUG,">$work_directory/DEBUG_VIEWER"); } 
if ($debug_viewer eq "Y") { print DEBUG "In FlowViewer.cgi\n"; } 

# Create the header portion of the FlowViewer webpage

print "Content-type:text/html\n\n";
print "<html>\n";
print "<head>\n";
print "<title>\n";
print "FlowViewer $version\n";
print "</title>\n";
print "</head>\n";
print "<body bgcolor=$bg_color text=$text_color link=$link_color vlink=$vlink_color>";
print "<center>";
print "<img src=$reports_short/FlowViewer.png border=0 usemap=#FlowViewer_Map>";
if ($user_logo ne "") { 
	print "<a href=$user_hyperlink>";
	print "<img src=$reports_short/$user_logo border=0>";
	print "</a>";
}
print "</center><b><pre>";

# Copy image into Reports directory

if (!-e $reports_directory) { 

        mkdir($reports_directory,$html_dir_perms) || die "cannot mkdir $reports_directory: $!";   
        chmod $html_dir_perms, $reports_directory;

	$copy_command = "cp $cgi_bin_directory/FlowViewer.png $reports_directory";
	system($copy_command);
        $icon_png_file = "$reports_directory/FlowViewer.png";
	chmod $html_dir_perms, $icon_png_file;

	$copy_command = "cp $cgi_bin_directory/FlowViewer_Save.png $reports_directory";
	system($copy_command);
        $icon_png_file = "$reports_directory/FlowViewer_Save.png";
	chmod $html_dir_perms, $icon_png_file;

	print "<pre><b>";
        print "      The directory for storing FlowViewer files has been created:\n\n"; 
        print "      <i>$reports_directory</i>\n\n"; 
        print "      Please ensure this directory has adequate permissions for your\n"; 
        print "      web server process owner (e.g., 'apache') to write into it.\n\n"; 
	print "\n";
        print "      <a href=$cgi_bin_short/FlowViewer.cgi>RETURN</a>\n"; 
        print "</pre></b>"; 
     
        exit; 
}

$copy_command = "cp $cgi_bin_directory/FlowViewer.png $reports_directory";
system($copy_command);

# Set up start and end times according to configurable start and end offsets  
      
($sec,$min,$hr,$date,$mnth,$yr,$day,$yr_date,$DST) = localtime(time-$start_offset); 
$mnth += 1; 
$yr   += 1900; 
$min_delta = $min % 5; 
$min = $min - $min_delta; 
if (length $min < 2) { $min = "0" . $min; }  
$start_date = $mnth ."/". $date ."/". $yr; 
if ($use_even_hours eq "Y") {
	$start_time = $hr .":00:00"; }
else {
	$start_time = $hr .":". $min .":00"; }
      
($sec,$min,$hr,$date,$mnth,$yr,$day,$yr_date,$DST) = localtime(time-$end_offset); 
$mnth += 1; 
$yr   += 1900; 
$min_delta = $min % 5; 
$min = $min - $min_delta; 
if (length $min < 2) { $min = "0" . $min; }  
$end_date = $mnth ."/". $date ."/". $yr;
if ($use_even_hours eq "Y") {
	$end_time = $hr .":00:00"; }
else {
	$end_time = $hr .":". $min .":00"; }

# If a user has called this program from another FlowViewer tool, pre-fill the input fields
 
$query_string = $ENV{'QUERY_STRING'};
 
if ($query_string ne "") {
        $query_string =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/ge ;
        chop $query_string;
        @fields = split(/&/,$query_string); 
        foreach $input_field (@fields) { 
                ($field,$field_value) = split(/=/,$input_field); 
                if ($field eq "filter_hash")         { $filter_hash = $field_value; } 
                if ($field eq "device_name")         { $selected_device_name = $field_value; } 
                if ($field eq "start_date")          { $start_date = $field_value; } 
                if ($field eq "start_time")          { $start_time = $field_value; } 
                if ($field eq "end_date")            { $end_date = $field_value; } 
                if ($field eq "end_time")            { $end_time = $field_value; } 
                if ($field eq "source_addresses")    { $source_address = $field_value; } 
                if ($field eq "source_ports")        { $source_port = $field_value; } 
                if ($field eq "source_ifs")          { $source_if = $field_value; } 
                if ($field eq "sif_names")           { $selected_sif_name = $field_value; } 
                if ($field eq "source_ases")         { $source_as = $field_value; } 
                if ($field eq "dest_addresses")      { $dest_address = $field_value; } 
                if ($field eq "dest_ports")          { $dest_port = $field_value; } 
                if ($field eq "dest_ifs")            { $dest_if = $field_value; } 
                if ($field eq "dif_names")           { $selected_dif_name = $field_value; } 
                if ($field eq "dest_ases")           { $dest_as = $field_value; } 
                if ($field eq "protocols")           { $protocols = $field_value; } 
                if ($field eq "tos_fields")          { $tos_fields = $field_value; } 
                if ($field eq "tcp_flags")           { $tcp_flags = $field_value; } 
                if ($field eq "exporter")            { $selected_exporter = $field_value; } 
                if ($field eq "nexthop_ips")         { $nexthop_ip = $field_value; } 
                if ($field eq "tracking_label")      { $tracking_label = $field_value; } 
                if ($field eq "general_comment")     { $general_comment = $field_value; } 
                if ($field eq "sampling_multiplier") { $sampling_multiplier = $field_value; } 
        }        
} 

$cutoff_lines = 100;
$sort_field   = 4;

if ($selected_device_name ne "") {
	$interfaces_file = "$cgi_bin_directory/NamedInterfaces_Devices";
} elsif ($selected_exporter ne "") {
	$interfaces_file = "$cgi_bin_directory/NamedInterfaces_Exporters";
}

$pulldown_width = 140;

print "<form action=$cgi_bin_short/FlowViewer_Main.cgi method=POST>";
print "Filter Criteria:";

# Print out Saved Filters

$saved_filters_file = "$cgi_bin_directory/FlowViewer_SavedFilters";
if (-e $saved_filters_file) {
	print " <select name=node onchange=window.location.href=this[selectedIndex].value\; style=\"width: $pulldown_width\">\n";
	if ($filter_hash eq "") { print "<option selected>Select Saved Filter"; }
	open (FILTERS,"<$saved_filters_file");
	while (<FILTERS>) {
	        chomp;
	        ($existing_hash,$existing_name,$existing_string) = split(/\^/);
	        $option_value = "$cgi_bin_short/FlowViewer.cgi?filter_hash=$existing_hash&$existing_string";
	        if ($existing_hash eq $filter_hash) {
	                print "<option selected value=$option_value>$existing_name";
	                $current_hash = $existing_hash;
	        } else {
	                print "<option value=$option_value>$existing_name";
	        }
	}
	close FILTERS;
	$option_value = "$cgi_bin_short/FlowViewer_Save.cgi?name_filter^$current_hash";
	print "<option value=$option_value>Review or Modify Filters";
	print "</select>";
}
print "\n\n";

if ($devices[0] ne "") {
        print "         Device: <select name=device_name onchange=window.location.href=this[selectedIndex].value\; style=\"width: $pulldown_width\">\n";
        print "<option selected value=\"\">Select Device\n";
        foreach $device_name (@devices) {
                $option_value = "$cgi_bin_short/FlowViewer.cgi?device_name=$device_name&";
                if ($device_name eq $selected_device_name) {
                        print "<option selected value=$option_value>$device_name";
                } else {
                        print "<option value=$option_value>$device_name";
                }
        }
        print "</select>";
}

if ($exporters[0] ne "") {
        if ($devices[0] ne "") { print "&nbsp&nbsp&nbsp&nbsp&nbsp"; }
        print "       Exporter: <select name=exporter onchange=window.location.href=this[selectedIndex].value\; style=\"width: $pulldown_width\">\n";
        print "<option selected value=\"\">Select Exporter\n";
        foreach $exporter_pair (@exporters) {
                ($exporter_ip,$exporter_name) = split(/:/,$exporter_pair);
                $option_value = "$cgi_bin_short/FlowViewer.cgi?exporter=$exporter_ip&";
                if ($exporter_ip eq $selected_exporter) {
                        $exporter = $exporter_ip;
                        print "<option selected value=$option_value>$exporter_name";
                } else {
                        print "<option value=$option_value>$exporter_name";
                }
        }
        print "</select>";
}

if (($devices[0] ne "") && ($exporters[0] eq "")) {
        print "      Next Hop IP: <input type=text size=19 name=nexthop_ip value=\"$nexthop_ip\">"; }
if (($devices[0] eq "") && ($exporters[0] ne "")) {
        print "    Next Hop IP: <input type=text size=19 name=nexthop_ip value=\"$nexthop_ip\">"; }
if (($devices[0] ne "") && ($exporters[0] ne "")) {
        print "        Next Hop IP: <input type=text size=19 name=nexthop_ip value=\"$nexthop_ip\">"; }
if (($devices[0] eq "") && ($exporters[0] eq "")) {
        print "    Next Hop IP: <input type=text size=19 name=nexthop_ip value=\"$nexthop_ip\">"; }
print "\n\n";

print "     Start Date: <input type=text size=10 name=start_date value=$start_date></b> (mm/dd/yyyy)<b>    Start Time: <input type=text size=10 name=start_time value=$start_time>  </b>(hh:mm:ss)<b>     TOS Field: <input type=text size=10 name=tos_fields value=\"$tos_fields\">  </b>(e.g., -0x0b/0x0F)<b>\n";
print "\n";
print "       End Date: <input type=text size=10 name=end_date value=$end_date></b> (mm/dd/yyyy)<b>      End Time: <input type=text size=10 name=end_time value=$end_time>  </b>(hh:mm:ss)<b>      TCP Flag: <input type=text size=10 name=tcp_flags value=\"$tcp_flags\">    Protocol: <input type=text size=6 name=protocols value=\"$protocols\">\n";
print "\n";

# Print out pulldown with named Source interfaces

print "      Source IP: <input type=text size=19 name=source_address value=\"$source_address\">";
print "    Source Interface: <input type=text size=2 name=source_if value=\"$source_if\">";

# Print out Include named interfaces

open (NAMED,"<$interfaces_file");
chomp (@interfaces = <NAMED>);
close (NAMED);

print "  <select name=sif_name width=20></b>\n";
print "<option selected value=\"\">Interface Names\n";
print "<optgroup label=\"Include ...\">";
foreach $interface (@interfaces) {
        if (($interface eq "") || (substr($interface,0,1) eq "#")) { next; }
        ($device,$interface_index,$interface_name) = split(/:/,$interface);
        if (($device eq $selected_device_name) || ($device eq $selected_exporter)) {
                if ($interface_index eq $selected_sif_name) {
                        print "<option selected value=$interface_index>$interface_index $interface_name\n";
                } else {
                        print "<option value=$interface_index>$interface_index $interface_name\n";
                }
        }
}

# Print out Exclude named interfaces

print "<optgroup label=\"Exclude ...\">";
foreach $interface (@interfaces) {
        if ($interface eq "") { next; }
        ($device,$interface_index,$interface_name) = split(/:/,$interface);
        $interface_index *= -1;
        if (($device eq $selected_device_name) || ($device eq $selected_exporter)) {
                if ($interface_index eq $selected_sif_name) {
                        print "<option selected value=$interface_index>$interface_index $interface_name\n";
                } else {
                        print "<option value=$interface_index>$interface_index $interface_name\n";
                }
        }
}

print "</select>";
print "       Source Port: <input type=text size=5 name=source_port value=\"$source_port\">";
print "   Source AS: <input type=text size=6 name=source_as value=\"$source_as\">\n\n";

# Print out pulldown with named Destination interfaces

print "        Dest IP: <input type=text size=19 name=dest_address value=\"$dest_address\">";
print "      Dest Interface: <input type=text size=2 name=dest_if value=\"$dest_if\">";

# Print out Include named interfaces

open (NAMED,"<$interfaces_file");
chomp (@interfaces = <NAMED>);
close (NAMED);

print "  <select name=dif_name width=20></b>\n";
print "<option selected value=\"\">Interface Names\n";
print "<optgroup label=\"Include ...\">";
foreach $interface (@interfaces) {
        if ($interface eq "") { next; }
        ($device,$interface_index,$interface_name) = split(/:/,$interface);
        if (($device eq $selected_device_name) || ($device eq $selected_exporter)) {
                if ($interface_index eq $selected_dif_name) {
                        print "<option selected value=$interface_index>$interface_index $interface_name\n";
                } else {
                        print "<option value=$interface_index>$interface_index $interface_name\n";
                }
        }
}

# Print out Exclude named interfaces

print "<optgroup label=\"Exclude ...\">";
foreach $interface (@interfaces) {
        if ($interface eq "") { next; }
        ($device,$interface_index,$interface_name) = split(/:/,$interface);
        $interface_index *= -1;
        if (($device eq $selected_device_name) || ($device eq $selected_exporter)) {
                if ($interface_index eq $selected_dif_name) {
                        print "<option selected value=$interface_index>$interface_index $interface_name\n";
                } else {
                        print "<option value=$interface_index>$interface_index $interface_name\n";
                }
        }
}

print "</select>";
print "         Dest Port: <input type=text size=5 name=dest_port value=\"$dest_port\">";
print "     Dest AS: <input type=text size=6 name=dest_as value=\"$dest_as\">\n";

print "\n";
print "<center></b>Notes: Multiple field entries, separated by commas, are permitted in the fields above.\n";
print "A minus sign (-) will negate an entry (e.g. -1776 for AS, would mean any AS but 1776).\n";
print "IP fields accept networks (e.g. 192.168.10.0/19), hosts, and names (e.g. www.abc.com).<b></center>";
print "\n";
print "<hr size=4>";
print "Reporting Parameters:\n\n";
print " Statistics: <select name=stat_report width=16></b>\n";
print "<option value=0 selected>Statistics Reports\n";
print "<option value=99>Summary\n";
print "<option value=5>UDP/TCP Destination Port\n";
print "<option value=6>UDP/TCP Source Port\n";
print "<option value=7>UDP/TCP Port\n";
print "<option value=8>Destination IP\n";
print "<option value=9>Source IP\n";
print "<option value=10>Source/Destination IP\n";
print "<option value=11>Source or Destination IP\n";
print "<option value=12>IP Protocol\n";
print "<option value=17>Input Interface\n";
print "<option value=18>Output Interface\n";
print "<option value=23>Input/Output Interface\n";
print "<option value=19>Source AS\n";
print "<option value=20>Destination AS\n";
print "<option value=21>Source/Destination AS\n";
print "<option value=22>IP ToS\n";
print "<option value=24>Source Prefix\n";
print "<option value=25>Destination Prefix\n";
print "<option value=26>Source/Destination Prefix\n";
print "<option value=27>Exporter IP\n";
print "</select>";
print "  Printed: <select name=print_report width=16></b>\n";
print "<option value=0 selected>Print Reports\n";
print "<option value=1>Flow Times\n";
print "<option value=4>AS Numbers\n";
print "<option value=5>132 Columns\n";
print "<option value=9>1 Line with Tags\n";
print "<option value=10>AS Aggregation\n";
print "<option value=11>Protocol Port Aggregation\n";
print "<option value=12>Source Prefix Aggregation\n";
print "<option value=13>Destination Prefix Aggregation\n";
print "<option value=14>Prefix Aggregation\n";
print "<option value=24>Full (Catalyst)\n";
print "</select>";
print "   Include Flow if: <select name=flow_select width=16></b>\n";
print "<option value=1 selected>Any Part in Specified Time Span\n";
print "<option value=2>End Time in Specified Time Span\n";
print "<option value=3>Start Time in Specified Time Span\n";
print "<option value=4>Entirely in Specified Time Span\n";
print "</select>";
print "  Sort Field: <input type=text size=3 name=sort_field value=$sort_field>";
print "\n\n";
print " Pie Charts: <select name=pie_charts></b>\n";
if ($pie_chart_default == 0) {
	print "    <option selected value=0>None\n";
	print "    <option value=1>With Others\n";
	print "    <option value=2>Without Others\n";
}
if ($pie_chart_default == 1) {
	print "    <option value=0>None\n";
	print "    <option selected value=1>With Others\n";
	print "    <option value=2>Without Others\n";
}
if ($pie_chart_default == 2) {
	print "    <option value=0>None\n";
	print "    <option value=1>With Others\n";
	print "    <option selected value=2>Without Others\n";
}
print "    </select>";
print "  Cutoff Lines: <input type=text size=3 name=cutoff_lines value=$cutoff_lines>";
print "  Cutoff Octets: <input type=text size=6 name=cutoff_octets value=$cutoff_octets>";
print "  Resolve Addresses: <select name=resolve_addresses width=4></b>\n";
print "    <option selected value=Y>Y\n";
print "    <option value=N>N\n";
print "    </select>";
print "  Oct Conv: <select name=unit_conversion width=4></b>\n"; 
print "    <option selected value=Y>Y\n"; 
print "    <option value=N>N\n"; 
print "    </select>";
print "  Sampling Multip.: <input type=text size=3 name=sampling_multiplier value=$sampling_multiplier>";
print "\n\n";
print "<hr size=4>";
print "\n";
print "<center>";
print "<input type=submit name=action value=\"Generate Report\">&nbsp\;<input type=reset value=\"Reset Values\">\n";
print "</center>";
print "</pre>";
print "</form>\n";
print "<map name=FlowViewer_Map>\n"; 
print "<area shape=rect coords=\"0,0,75,88\" href=$cgi_bin_short/FlowViewer.cgi>\n"; 
print "<area shape=rect coords=\"75,22,271,68\" href=$cgi_bin_short/FlowViewer.cgi>\n"; 
print "<area shape=rect coords=\"75,0,271,22\" href=$cgi_bin_short/FlowGrapher.cgi>\n"; 
print "<area shape=rect coords=\"75,68,271,88\" href=$cgi_bin_short/FlowTracker.cgi>\n"; 
print "</map>\n";
print "</body>\n";
print "</html>\n";
