#! /usr/bin/perl
#
#  Purpose:
#  FlowViewer_Save.cgi controls the moving of a temporary copy of the
#  FlowViewer and FlowGrapher HTML files and the FlowGrapher graph image 
#  file to either the FlowViewer or FlowGrapher htdocs directory. This
#  is to save these files for later use.
#
#  Description:
#
#  Input arguments (received from the form):
#  Name                 Description
#  -----------------------------------------------------------------------
#  save_filename        Name of temp file created by FlowViewer or FlowGrapher
#
#  Modification history:
#  Author       Date            Vers.   Description
#  -----------------------------------------------------------------------
#  J. Loiacono  07/04/2005      1.0     Original version.
#  J. Loiacono  01/01/2006      1.0     Updated for FlowGrapher
#  J. Loiacono  12/25/2006      3.1     [No changes to this module]
#  J. Loiacono  12/07/2007      3.3     Now handles saved filters
#
#$Author$
#$Date$
#$Header$
#
###########################################################################
#
#               BEGIN EXECUTABLE STATEMENTS
#
 
use FlowViewer_Configuration;
use FlowViewer_Utilities;
 
if ($debug_viewer eq "Y") { open (DEBUG,">>$work_directory/DEBUG_SAVE"); }

$saved_filters_file = "$cgi_bin_directory/FlowViewer_SavedFilters";

($action,$operand) = split(/\^/,$ENV{'QUERY_STRING'});
$action  =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/ge;
$operand =~ s/%([0-9A-Fa-f]{2})/chr(hex($1))/ge;

if ($action eq "") { $action = "save_filter"; }

if ($action eq "save_file") {

	$save_filename = $operand;
	
	if (!-e $save_directory) { 
	      
	        mkdir($save_directory,$html_dir_perms) || die "cannot mkdir $save_directory: $!"; 
	        chmod $html_dir_perms, $save_directory; 
	      
	        $copy_command = "cp $cgi_bin_directory/FlowGrapher.png $save_directory"; 
	        system($copy_command); 
	        $icon_png_file = "$save_directory/FlowGrapher.png";  
	        chmod $html_dir_perms, $icon_png_file;  
	      
	        $copy_command = "cp $cgi_bin_directory/FlowViewer.png $save_directory"; 
	        system($copy_command); 
	        $icon_png_file = "$save_directory/FlowViewer.png";  
	        chmod $html_dir_perms, $icon_png_file;  
	      
		print "Content-type:text/html\n\n";
		print "<head>";
		print "</head>";
		print "<body bgcolor=$bg_color text=$text_color link=$link_color vlink=$vlink_color>\n";
		print "<style type=text/css><!-- A { text-decoration:none } --></style>";
		print "<title>Creating FlowViewer Saves Directory</title>";

	        print "<pre><b>\n\n"; 
	        print "      The directory for saving FlowViewer and FlowGrapher reports has been created:\n\n"; 
	        print "      <i>$save_directory</i>\n\n"; 
	        print "      Please ensure this directory has adequate permissions for your\n";
	        print "      web server process owner (e.g., 'apache') to write into it.\n";
	        print "      A list of all saved files can be found at:\n\n"; 
		print "      <a href=$FlowViewer_service://$FlowViewer_server$save_short>FlowViewer Saved Files</a>\n\n";
	        print "      After clicking on above, please add the page to your bookmarks.\n";
	        print "</pre></b>";

		$created_directory = 1;
	}

	if ($save_filename =~ /graph/) {

		($file_name,$suffix) = split(/\.html/,$save_filename);

		$temp_file = "$work_directory/$save_filename";
		open (TEMP,"<$temp_file");
		open (SAVE,">$save_directory/$save_filename");
		while (<TEMP>) {
			if (/$file_name/) { 
				$line = $_;
				$line =~ s/$graphs_short/$save_short/;
				print SAVE $line;
			} else {
				print SAVE $_;
			}
		}
		close TEMP;
		close SAVE;

		$save_graphname = $file_name .".png";
		$copy_command = "cp $graphs_directory/$save_graphname $save_directory/$save_graphname";
		system($copy_command);

		$saved_page = "$FlowViewer_service://$FlowViewer_server$save_short/$save_filename";

	} elsif ($save_filename ne "") {
	
		$move_command = "mv $work_directory/$save_filename $save_directory/$save_filename";
		system($move_command);
		$saved_page = "$FlowViewer_service://$FlowViewer_server$save_short/$save_filename";
	}

	open (HTML,">$save_directory/index.html"); 

	print HTML "<html>\n"; 
	print HTML "<head>\n"; 
	print HTML "<title>\n"; 
	print HTML "FlowViewer Saved Reports and Graphs\n"; 
	print HTML "</title>\n"; 
	print HTML "</head>\n"; 
	print HTML "<body bgcolor=$bg_color text=$text_color link=$link_color vlink=$vlink_color>\n"; 
	print HTML "<style type=text/css><!-- A { text-decoration:none } --></style>"; 
	print HTML "<center>\n"; 
	print HTML "<img src=FlowViewer.png border=0 usemap=#FlowViewer_Map>";  
	if ($user_logo ne "") {    
	        print HTML "<a href=$user_hyperlink>";  
	        print HTML "<img src=$reports_short/$user_logo border=0>";
	        print HTML "</a>";
	}    

	print HTML "</center><b><pre>\n";
	print HTML "\n\n";
	print HTML "    Date       Time       Created By    Report Label</b>\n";

	# Output a list of Saved files

        while ($save_file = <$save_directory/*>) { 
                ($directory,$saved_label) = $save_file =~ m/(.*\/)(.*)$/;
		push (@saved_files,$saved_label);
	} 

	@sorted_saved_files = sort (@saved_files);

	foreach $saved_label (@sorted_saved_files) {
		if ($saved_label =~ /html/) { 
			if ($saved_label =~ /index/) { next; }
			($filename,$suffix) = split(/\.html/,$saved_label);
			($save_date,$save_time,$remainder) = split(/_/,$filename);
			$yr = substr($save_date,0,4); $yr -= 1900;
			$mn = substr($save_date,4,2); $mn -= 1;
			$da = substr($save_date,6,2); if (substr($da,0,1) eq "0") { $da = substr($da,1,1); }
			$hr = substr($save_time,0,2); if (substr($hr,0,1) eq "0") { $hr = substr($hr,1,1); }
			$mi = substr($save_time,2,2); if (substr($mi,0,1) eq "0") { $mi = substr($mi,1,1); }
			$sc = substr($save_time,4,2); if (substr($sc,0,1) eq "0") { $sc = substr($sc,1,1); }
			$date_time_out = format_date($sc,$mi,$hr,$da,$mn,$yr);

			$created_by = "FlowViewer ";
			if ($filename =~ /graph/) { $created_by = "FlowGrapher"; }

			if ($mn ne $last_mn) { print HTML "\n"; }

			print HTML "<b>    $date_time_out   $created_by   <a href=$save_short/$saved_label>$filename</a></b>\n";
			$last_mn = $mn;
		}
	}

	print HTML "<map name=FlowViewer_Map>\n";  
	print HTML "<area shape=rect coords=\"0,0,75,88\" href=$cgi_bin_short/FlowViewer.cgi>\n";  
	print HTML "<area shape=rect coords=\"75,22,271,68\" href=$cgi_bin_short/FlowViewer.cgi>\n";  
	print HTML "<area shape=rect coords=\"75,0,271,22\" href=$cgi_bin_short/FlowGrapher.cgi>\n";  
	print HTML "<area shape=rect coords=\"75,68,271,88\" href=$cgi_bin_short/FlowTracker.cgi>\n";  
	print HTML "</map>\n";  
	print HTML "</body>\n";
	print HTML "</html>\n";

	close HTML;

	if (!$created_directory) {
		print "Content-type:text/html\n\n";
		print "<head>";
		print "<META HTTP-EQUIV=Refresh CONTENT=\"0; URL=$saved_page\">";
		print "</head>";
		print "<body bgcolor=$bg_color>";
		print "<title>Save FlowViewer or FlowGrapher</title>";
	}
}

if (($action eq "name_filter") || ($action eq "remove_filter")) {

	if ($action eq "name_filter") {
		$key = substr($operand,0,8);
		if (($key eq "device_n") || ($key eq "exporter")) {
			$call_string = $operand;
			$button_text = "Save Filter";
		} elsif ($key eq "") {
			$button_text = "Save Filter";
		} else {
			$filter_hash = $operand;
			open (FILTERS,"<$saved_filters_file");
			while (<FILTERS>) {
				($existing_hash,$existing_name,$existing_string) = split(/\^/);
				if ($existing_hash eq $filter_hash) { 
					$call_string = $existing_string;
					$filter_name = $existing_name;
					$button_text = "Update Filter";
				}
			}
		}

	        @fields = split(/&/,$call_string); 
	        foreach $input_field (@fields) {
	                ($field,$field_value) = split(/=/,$input_field);
	                if ($field eq "device_name")         { $selected_device_name = $field_value; }
	                if ($field eq "start_date")          { $start_date = $field_value; }
	                if ($field eq "start_time")          { $start_time = $field_value; }
	                if ($field eq "end_date")            { $end_date = $field_value; }
	                if ($field eq "end_time")            { $end_time = $field_value; }
	                if ($field eq "source_addresses")    { $source_address = $field_value; }
	                if ($field eq "source_ports")        { $source_port = $field_value; }
	                if ($field eq "source_ifs")          { $source_if = $field_value; }
                	if ($field eq "sif_names")           { $selected_sif_name = $field_value; }
	                if ($field eq "source_ases")         { $source_as = $field_value; }
	                if ($field eq "dest_addresses")      { $dest_address = $field_value; }
	                if ($field eq "dest_ports")          { $dest_port = $field_value; }
	                if ($field eq "dest_ifs")            { $dest_if = $field_value; }
                	if ($field eq "dif_names")           { $selected_dif_name = $field_value; }
	                if ($field eq "dest_ases")           { $dest_as = $field_value; }
	                if ($field eq "protocols")           { $protocols = $field_value; }
	                if ($field eq "tos_fields")          { $tos_fields = $field_value; }
	                if ($field eq "tcp_flags")           { $tcp_flags = $field_value; }
	                if ($field eq "exporter")            { $selected_exporter = $field_value; }
	                if ($field eq "nexthop_ips")         { $nexthop_ip = $field_value; }
	                if ($field eq "sampling_multiplier") { $sampling_multiplier = $field_value; }
	        }

		if ($selected_device_name ne "") {
        		$interfaces_file = "$cgi_bin_directory/NamedInterfaces_Devices";
		} elsif ($selected_exporter ne "") {
        		$interfaces_file = "$cgi_bin_directory/NamedInterfaces_Exporters";
		}

	} elsif ($action eq "remove_filter") {

		$button_text = "Save Filter";

		$remove_hash = $operand;
		open (FILTERS,"<$saved_filters_file");
		@filters = <FILTERS>;
		close FILTERS;

		foreach $existing_filter (@filters) {
			chomp $existing_filter;
			($existing_hash,$existing_name,$existing_string) = split(/\^/,$existing_filter);
			if ($existing_hash eq $remove_hash) {
				next;
			} else {
				push (@new_filter_list,$existing_filter);
			}
		}

		open (FILTERS,">$saved_filters_file");
		foreach $new_filter (@new_filter_list) { print FILTERS "$new_filter\n"; }
		close FILTERS;
	}
	
        print "Content-type:text/html\n\n"; 
        print "<head>"; 
        print "<style type=text/css><!-- A { text-decoration:none } --></style>"; 
        print "</head>"; 
	print "<body bgcolor=$bg_color text=$text_color link=$link_color vlink=$vlink_color>\n";
        print "<title>FlowViewer $version Name Filter for Saving</title>"; 
	print "<center>"; 
	print "<img src=$reports_short/FlowViewer.png border=0 usemap=#FlowViewer_Map>"; 
	if ($user_logo ne "") {  
	        print "<a href=$user_hyperlink>"; 
	        print "<img src=$reports_short/$user_logo border=0>"; 
	        print "</a>";  
	}  
	print "</center>"; 

	print "<b>";
	if ($filter_name ne "") { 
		print "<center><h3><font face=arial>Saved Filters</font></h3></center>\n";
		print "<pre>";
		print "  Filter Criteria: <font face=arial>$filter_name</font><br>\n";
	} else {
		print "<center><h3><font face=arial>Saved Filters</font></h3></center>\n";
		print "<pre>";
		print "  Filter Criteria:\n\n";
	}
	print "<form action=$cgi_bin_short/FlowViewer_Save.cgi method=POST>";
	 
	if ($devices[0] ne "") {
	        print "           Device: <select name=device_name onchange=window.location.href=this[selectedIndex].value\; style=\"width: $pulldown_width\">\n";
	        print "<option selected value=\"\">Select Device\n";
	        foreach $device_name (@devices) {
	                $option_value = "$cgi_bin_short/FlowViewer_Save.cgi?name_filter^device_name=$device_name&";
	                if ($device_name eq $selected_device_name) {
	                        print "<option selected value=$option_value>$device_name";
	                } else {
	                        print "<option value=$option_value>$device_name";
	                }
	        }
	        print "</select>";
	}
	
	if ($exporters[0] ne "") {
	        if ($devices[0] ne "") { print "&nbsp&nbsp&nbsp&nbsp&nbsp"; }
	        print "         Exporter: <select name=exporter onchange=window.location.href=this[selectedIndex].value\; style=\"width: $pulldown_width\">\n";
	        print "<option selected value=\"\">Select Exporter\n";
	        foreach $exporter_pair (@exporters) {
	                ($exporter_ip,$exporter_name) = split(/:/,$exporter_pair);
	                $option_value = "$cgi_bin_short/FlowViewer_Save.cgi?name_filter^exporter=$exporter_ip&";
	                if ($exporter_ip eq $selected_exporter) {
	                        $exporter = $exporter_ip;
	                        print "<option selected value=$option_value>$exporter_name";
	                } else {
	                        print "<option value=$option_value>$exporter_name";
	                }
	        }
	        print "</select>";
	}
	 
	if (($devices[0] ne "") && ($exporters[0] eq "")) {
	        print "      Next Hop IP: <input type=text size=19 name=nexthop_ip value=\"$nexthop_ip\">"; }
	if (($devices[0] eq "") && ($exporters[0] ne "")) {
	        print "    Next Hop IP: <input type=text size=19 name=nexthop_ip value=\"$nexthop_ip\">"; }
	if (($devices[0] ne "") && ($exporters[0] ne "")) {
	        print "        Next Hop IP: <input type=text size=19 name=nexthop_ip value=\"$nexthop_ip\">"; }
	if (($devices[0] eq "") && ($exporters[0] eq "")) {
	        print "    Next Hop IP: <input type=text size=19 name=nexthop_ip value=\"$nexthop_ip\">"; }
	print "\n\n";
	 
	print "        TCP Flags: <input type=text size=10 name=tcp_flags value=\"$tcp_flags\">";
	print "           TOS Fields: <input type=text size=10 name=tos_fields value=\"$tos_fields\">  </b>(e.g., -0x0b/0x0F)<b>";
	print "             Protocols: <input type=text size=6 name=protocols value=\"$protocols\">";
	print "    Sampling: <input type=text size=3 name=sampling_multiplier value=$sampling_multiplier>";
	print "\n\n";

	# Print out pulldown with named Source interfaces
	
	print "        Source IP: <input type=text size=19 name=source_address value=\"$source_address\">";
	print "    Source Interface: <input type=text size=2 name=source_if value=\"$source_if\">";

	# Print out Include named interfaces

	open (NAMED,"<$interfaces_file");
	chomp (@interfaces = <NAMED>);
	close (NAMED);

	print "  <select name=sif_name width=20></b>\n";
	print "<option selected value=\"\">Interface Names\n";
	print "<optgroup label=\"Include ...\">";
	foreach $interface (@interfaces) {
	        if ($interface eq "") { next; }
	        ($device,$interface_index,$interface_name) = split(/:/,$interface);
	        if (($device eq $selected_device_name) || ($device eq $selected_exporter)) {
	                if ($interface_index eq $selected_sif_name) {
	                        print "<option selected value=$interface_index>$interface_index $interface_name\n";
	                } else {
	                        print "<option value=$interface_index>$interface_index $interface_name\n";
			}
		}
	}

	# Print out Exclude named interfaces

	print "<optgroup label=\"Exclude ...\">";
	foreach $interface (@interfaces) {
	        if ($interface eq "") { next; }
	        ($device,$interface_index,$interface_name) = split(/:/,$interface);
	        $interface_index *= -1;
	        if (($device eq $selected_device_name) || ($device eq $selected_exporter)) {
	                if ($interface_index eq $selected_sif_name) {
	                        print "<option selected value=$interface_index>$interface_index $interface_name\n";
	                } else {
	                        print "<option value=$interface_index>$interface_index $interface_name\n";
			}
		}
	}

	print "</select>";
	print "      Source Port: <input type=text size=5 name=source_port value=\"$source_port\">";
	print "    Source AS: <input type=text size=6 name=source_as value=\"$source_as\">\n\n";

	# Print out pulldown with named Destination interfaces
	
	print "          Dest IP: <input type=text size=19 name=dest_address value=\"$dest_address\">";
	print "      Dest Interface: <input type=text size=2 name=dest_if value=\"$dest_if\">";

	# Print out Include named interfaces

	open (NAMED,"<$interfaces_file");
	chomp (@interfaces = <NAMED>);
	close (NAMED);

	print "  <select name=dif_name width=20></b>\n";
	print "<option selected value=\"\">Interface Names\n";
	print "<optgroup label=\"Include ...\">";
	foreach $interface (@interfaces) {
	        if ($interface eq "") { next; }
	        ($device,$interface_index,$interface_name) = split(/:/,$interface);
	        if (($device eq $selected_device_name) || ($device eq $selected_exporter)) {
	                if ($interface_index eq $selected_dif_name) {
	                        print "<option selected value=$interface_index>$interface_index $interface_name\n";
	                } else {
	                        print "<option value=$interface_index>$interface_index $interface_name\n";
			}
		}
	}

	# Print out Exclude named interfaces

	print "<optgroup label=\"Exclude ...\">";
	foreach $interface (@interfaces) {
	        if ($interface eq "") { next; }
	        ($device,$interface_index,$interface_name) = split(/:/,$interface);
	        $interface_index *= -1;
	        if (($device eq $selected_device_name) || ($device eq $selected_exporter)) {
	                if ($interface_index eq $selected_dif_name) {
	                        print "<option selected value=$interface_index>$interface_index $interface_name\n";
	                } else {
	                        print "<option value=$interface_index>$interface_index $interface_name\n";
			}
		}
	}

	print "</select>";
	print "        Dest Port: <input type=text size=5 name=dest_port value=\"$dest_port\">";
	print "      Dest AS: <input type=text size=6 name=dest_as value=\"$dest_as\">\n";

	print "\n";
	print "<center></b>Notes: Multiple field entries, separated by commas, are permitted in the fields above.\n"; 
	print "A minus sign (-) will negate an entry (e.g. -1776 for AS, would mean any AS but 1776).\n"; 
	print "IP fields accept networks (e.g. 192.168.10.0/19), hosts, and names (e.g. www.abc.com).<b></center>";
	print "\n";
	print "<center>";
	print "<b>Filter Name  </b><input type=text size= 30 name=filter_name value=\"$filter_name\">";
	print "  <input type=submit name=action value=\"$button_text\">";
	print "</center>";
	print "\n";
	print "<hr size=4>";
	
	# List existing filter names for user information

	print "\n    Existing Filters (click to Review or Modify)\n\n";
	open (FILTERS,"<$saved_filters_file");
	while (<FILTERS>) {
		($existing_hash,$existing_name,$existing_string) = split(/\^/);

                $filter_out = $existing_name; 
                $length = length($filter_out); 
                for ($i=$length;$i<=60;$i++) { $filter_out .= " "; }

		$filter_link = "<a href=$cgi_bin_short/FlowViewer_Save.cgi?name_filter^$existing_hash>$filter_out</a>";
                $remove_link = "<a href=$cgi_bin_short/FlowViewer_Save.cgi?remove_filter^$existing_hash>Remove</a>";
		printf "      %-56s    %-8s\n", $filter_link, $remove_link;
	}
	print "</pre>";
	print "</form>\n";
	print "<map name=FlowViewer_Map>\n";
	print "<area shape=rect coords=\"0,0,75,88\" href=$cgi_bin_short/FlowViewer.cgi>\n";
	print "<area shape=rect coords=\"75,22,271,68\" href=$cgi_bin_short/FlowViewer.cgi>\n";
	print "<area shape=rect coords=\"75,0,271,22\" href=$cgi_bin_short/FlowGrapher.cgi>\n";
	print "<area shape=rect coords=\"75,68,271,88\" href=$cgi_bin_short/FlowTracker.cgi>\n";
	print "</map>\n";
	print "</body>\n";
	print "</html>\n";
}

if ($action eq "save_filter") {

	read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
	@pairs = split(/&/, $buffer);
	foreach $pair (@pairs) {
	    ($name, $value) = split(/=/, $pair);
	    $value =~ tr/+/ /;
	    $value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C",hex($1))/eg;
	    $FORM{$name} = $value;
	}
	 
	# Clean up input
	 
	$FORM{source_address} =~ s/\s+//g; $FORM{source_address} =~ s/,/, /g;
	$FORM{source_port}    =~ s/\s+//g; $FORM{source_port}    =~ s/,/, /g;
	$FORM{source_if}      =~ s/\s+//g; $FORM{source_if}      =~ s/,/, /g;
	$FORM{sif_name}       =~ s/\s+//g; $FORM{sif_name}      =~ s/,/, /g;
	$FORM{source_as}      =~ s/\s+//g; $FORM{source_as}      =~ s/,/, /g;
	$FORM{dest_address}   =~ s/\s+//g; $FORM{dest_address}   =~ s/,/, /g;
	$FORM{dest_port}      =~ s/\s+//g; $FORM{dest_port}      =~ s/,/, /g;
	$FORM{dest_if}        =~ s/\s+//g; $FORM{dest_if}        =~ s/,/, /g;
	$FORM{dif_name}       =~ s/\s+//g; $FORM{dif_name}      =~ s/,/, /g;
	$FORM{dest_as}        =~ s/\s+//g; $FORM{dest_as}        =~ s/,/, /g;
	$FORM{protocols}      =~ s/\s+//g; $FORM{protocols}      =~ s/,/, /g;
	$FORM{tos_fields}     =~ s/\s+//g; $FORM{tos_fields}     =~ s/,/, /g;
	$FORM{tcp_flags}      =~ s/\s+//g; $FORM{tcp_flags}      =~ s/,/, /g;
	$FORM{nexthop_ip}     =~ s/\s+//g; $FORM{nexthop_ip}     =~ s/,/, /g;
	 
	# Recover from Javascript links used to load named interfaces

	($link,$FORM{device_name}) = split(/=/,$FORM{device_name}); 
	($FORM{device_name},$tail) = split(/\&/,$FORM{device_name});
	($link,$FORM{exporter})    = split(/=/,$FORM{exporter}); 
	($FORM{exporter},$tail)    = split(/\&/,$FORM{exporter});

	# Parameters for generating a FlowViewer report
	 
	$device_name         = $FORM{'device_name'};
	$source_addresses    = $FORM{'source_address'};
	$source_ports        = $FORM{'source_port'};
	$source_ifs          = $FORM{'source_if'};
	$sif_names           = $FORM{'sif_name'};
	$source_ases         = $FORM{'source_as'};
	$dest_addresses      = $FORM{'dest_address'};
	$dest_ports          = $FORM{'dest_port'};
	$dest_ifs            = $FORM{'dest_if'};
	$dif_names           = $FORM{'dif_name'};
	$dest_ases           = $FORM{'dest_as'};
	$protocols           = $FORM{'protocols'};
	$tcp_flags           = $FORM{'tcp_flags'};
	$tos_fields          = $FORM{'tos_fields'};
	$exporter            = $FORM{'exporter'};
	$nexthop_ips         = $FORM{'nexthop_ip'};
	$sampling_multiplier = $FORM{'sampling_multiplier'};
	$filter_name         = $FORM{'filter_name'};
	 
	# Build call string to provide filter parameters to either FlowGrapher or FlowTracker
	 
	$call_string  = "device_name=$device_name&";
	$call_string .= "source_addresses=$source_addresses&";
	$call_string .= "source_ports=$source_ports&";
	$call_string .= "source_ifs=$source_ifs&";
	$call_string .= "sif_names=$sif_names&";
	$call_string .= "source_ases=$source_ases&";
	$call_string .= "dest_addresses=$dest_addresses&";
	$call_string .= "dest_ports=$dest_ports&";
	$call_string .= "dest_ifs=$dest_ifs&";
	$call_string .= "dif_names=$dif_names&";
	$call_string .= "dest_ases=$dest_ases&";
	$call_string .= "protocols=$protocols&";
	$call_string .= "tcp_flags=$tcp_flags&";
	$call_string .= "tos_fields=$tos_fields&";
	$call_string .= "exporter=$exporter&";
	$call_string .= "nexthop_ips=$nexthop_ips&";
	$call_string .= "sampling_multiplier=$sampling_multiplier&";
	$call_string =~ s/\s+//g;
	
        $filter_hash = $filter_name;
        $filter_hash =~ s/^\s+//;
        $filter_hash =~ s/\s+$//;
        $filter_hash =~ s/\&/-/g;
        $filter_hash =~ s/\//-/g;
        $filter_hash =~ s/\(/-/g;
        $filter_hash =~ s/\)/-/g;
        $filter_hash =~ s/\./-/g;
        $filter_hash =~ s/\s+/_/g;
        $filter_hash =~ tr/[A-Z]/[a-z]/;

	$button_text = "Save Filter";

        if (!-e $saved_filters_file) { 
		open (FILTERS,">$saved_filters_file");
		print FILTERS $filter_hash ."^". $filter_name ."^". $call_string;
		close FILTERS;
		$new_file = 1;
	} else {
		$new_filter = $filter_hash ."^". $filter_name ."^". $call_string;

		open (FILTERS,"<$saved_filters_file");
		@filters = <FILTERS>;
		close FILTERS;

		foreach $existing_filter (@filters) {
			chomp $existing_filter;
			($existing_hash,$existing_name,$existing_string) = split(/\^/,$existing_filter);
			if ($filter_hash eq $existing_hash) {
				push (@new_filter_list,$new_filter);
				$replacement = 1;
			} else {
				push (@new_filter_list,$existing_filter);
			}
		}
		if (!$replacement) { push (@new_filter_list,$new_filter); }

		@sorted_filters = sort (@new_filter_list);
			
		open (FILTERS,">$saved_filters_file");
		foreach $new_filter (@sorted_filters) { print FILTERS "$new_filter\n"; }
		close FILTERS;
	}

        print "Content-type:text/html\n\n"; 
        print "<head>"; 
        print "<style type=text/css><!-- A { text-decoration:none } --></style>"; 
        print "</head>"; 
        print "<body bgcolor=$bg_color>"; 
        print "<title>FlowViewer $version Name Filter for Saving</title>"; 
	print "<center>"; 
	print "<img src=$reports_short/FlowViewer.png border=0 usemap=#FlowViewer_Map>"; 
	if ($user_logo ne "") {  
	        print "<a href=$user_hyperlink>"; 
	        print "<img src=$reports_short/$user_logo border=0>"; 
	        print "</a>";  
	}  
	print "</center>"; 
	print "<pre>";
        print "\n\n";  
        print "<b>\n"; 
        if ($new_file) {
                chmod $saved_filters_perms, $saved_filters_file;
                print "      The file for storing Saved Filters has been created:\n\n"; 
                print "      <i>$saved_filters_file</i>\n\n"; 
        } 
	print "      Your filter <i>\"$filter_name\"</i> has been added (or updated) on the list of saved filters.\n\n";
	print "      You will be able to delete or modify this filter by choosing the\n";
	print "      \"Review or Modify Filters\" option on the Saved Filters pulldown.\n";
	print "\n";
	print "      If you've come from a report, use your browser's back button twice to return there.\n";
	print "\n";
	print "      <a href=$cgi_bin_short/FlowViewer_Save.cgi?name_filter^^>All Filters</a>\n";

	print "<map name=FlowViewer_Map>\n";  
	print "<area shape=rect coords=\"0,0,75,88\" href=$cgi_bin_short/FlowViewer.cgi?$call_string>\n";  
	print "<area shape=rect coords=\"75,22,271,68\" href=$cgi_bin_short/FlowViewer.cgi?$call_string>\n";  
	print "<area shape=rect coords=\"75,0,271,22\" href=$cgi_bin_short/FlowGrapher.cgi?$call_string>\n";  
	print "<area shape=rect coords=\"75,68,271,88\" href=$cgi_bin_short/FlowTracker.cgi?$call_string>\n";  
	print "</map>\n"; 
}
